#!/bin/csh -f
# this script run the gateway command
#
if ( $#argv < 2 ) then
	echo "Usage gateway_iface <action> <Parameters File>"
	exit 0
endif

# ARGS
set action = $argv[1]
set paramsFile = $argv[2] 

if ( $action == "" || $paramsFile == "" ) then
	echo "Usage gateIface <action> <paramsfile>"
	exit -1
endif

# LOCALS
if ($GENESIS_EDIR =~ /* || $GENESIS_EDIR =~ ?:*) then
	set _genesis_edir = $GENESIS_EDIR
else
	set _genesis_edir = $GENESIS_DIR/$GENESIS_EDIR
endif


if ( -f $paramsFile ) then
	source $paramsFile
else if ( $action == "sync" ) then
	echo "file $paramsFile doesn't exist"
	exit -1
endif

set ALL_ERR = $_genesis_edir/all/errs/ALL_ERRS
set host_conf_file = $GENESIS_DIR/hosts/${host}/communicator.ini
set scripts_dir = ${_genesis_edir}/all/odbcomm

alias get_param 'set \!:1 = `cat $host_conf_file | grep "\!:1=" | cut -d "=" -f1`'

if ($?GENESIS_TMP) then
   set temp_dir = $GENESIS_TMP
else if ($?GENESIS_DIR) then
   set temp_dir = $GENESIS_DIR/tmp
else
   set temp_dir = /genesis/tmp
endif

set resp_file = $temp_dir/resp.$$


set OS = (`uname -s | tr '[:lower:]' '[:upper:]'`)
set EXT = ""
if ( $OS =~ *WIN* ) set EXT = ".exe"

if ( ! -f "$_genesis_edir/misc/gateway${EXT}" ) then
	echo "Can't run gateway (GENESIS_EDIR=$_genesis_edir)"
	exit -1
endif

if ( $action == "sync" ) then
	set script = "$scripts_dir/sync"
else if ( $action == "grab" ) then
	set script = "$scripts_dir/grab"
endif

set users = (`"$_genesis_edir/misc/gateway${EXT}" WHO`)
if ( $status != 0 ) then
	echo "Error runinng gateway"
endif

set exist = "no"
foreach name ($users) 
	if ( $name == "$user@$host.$display" ) then
		set exist = "yes"
		if ( $action == "existence_only" ) then
			echo "true"
			rm -f $paramsFile
			exit 0
		endif
	endif
end

if ( $exist == "no" && $action == "existence_only" ) then
	echo false
	rm -f $paramsFile
	exit -1
endif

if ( $#users == 0 ) then
	echo "There is no users connected to the GND (GENESIS_DIR=$GENESIS_DIR)"
	exit -1
endif

if ( $exist == "no" ) then
	echo "The user : $user@$host.$display doesn't connect to the gnd"
	exit -1
endif

"$_genesis_edir/misc/gateway${EXT}" ${user}@${host}.${display} "COM script_run,name=$script,params=${paramsFile}" > $resp_file

set stat = (`cat $resp_file`)
set desc = (`cat $ALL_ERR | grep '\${$stat}' | cut -d\$ -f3`)

rm -f $resp_file

if ($stat != 0) then
	if ($stat == 8008) then # command not found
		echo "Can't grab picture, Check if the editor window is opened"
	else
		set res = `echo $desc | wc -l`
		if ($res > 1) then
			"Internal Error"
		else
			echo $desc
		endif
	endif
else
	echo "ok"
endif

rm -f $paramsFile
exit {$stat}
