#---------------------------------------------------
#
#  Module : install.tcl
#  Author : DK
#  Date   : 2013-07-09
#
#  Description :
#
#  This is the main module for the Frontline Pro-Installer.
#
# Version 3.3  04 Dec 2003
# Version 4.0  05 Nov 2009 Yefim
#		Build from perforce. New literature (brochures).
#		Avoid crash while saving default path and GUI parameters.
#---------------------------------------------------

# Global variables which can be overwritten by the defaults file
#
global VPIG
   
set VPIG(VERSION)		"2000 V10.0"
set VPIG(SOURCE)		"CDROM"
set VPIG(CDROM_PATH)		""
set VPIG(FTP_SERVER)		"ftp-us.frontline-pcb.com"
set VPIG(DAT_DEVICE)		"/dev/rmt/0m"
set VPIG(DISK_PATH)		""
set VPIG(PRODUCTS_OR_BEANS)	"products"
set VPIG(NAME) 			""
set VPIG(NAME_VERSION) 		""
set VPIG(NAME_OS)      		""
set VPIG(TOOLS_PATH)		""
set VPIG(INSTALL_PATH)		""
set VPIG(PRODUCT_PATH)		""
set VPIG(DEFAULTS_PATH)		""
set VPIG(LOG_PATH)		""
set VPIG(INSTALL_STATE)	        0
set VPIG(W)                     ".w"   
set VPIG(WMAIN)			".w.main"
set VPIG(WPLUGIN)		".w.plugin"
set VPIG(WPLUGIN_EMPTY)		".w.plugin.empty"
set VPIG(WPLUGIN_FRAME)		".w.plugin.frame"
set VPIG(WMESSAGE)		".wmessage"
set VPIG(FTP_USER)		"ftp"
set VPIG(FTP_LOCATIONS)       \
   { "ftp-us.frontline-pcb.com" "Frontline mirror (U.S.)" "/pub/release" \
     "ftp.frontline-pcb.com"    "Frontline R&D (Israel)"  "/pub/release" }
#

proc vpi.main {} {     
  global VPIG 
  
  tclu_set_tools_dir $VPIG(TOOLS_PATH)
  tclu_set_os $VPIG(OS)
  tku_set_tools_dir $VPIG(TOOLS_PATH)
  tku_set_os $VPIG(OS)
  vpi.set_unix_commands  
  
  set VPIG(USER_DIR)      [file join [tclu_get_home] .valor_install]
  set VPIG(DEFAULTS_PATH) [file join $VPIG(USER_DIR) defaults]  
  set VPIG(LOG_PATH)      [file join $VPIG(USER_DIR) log]  
  set VPIG(C_BG)	  [c_dblue]
  
  catch {file mkdir $VPIG(USER_DIR)}

  tclu_log_set_path $VPIG(LOG_PATH)
  tclu_log_newline 
  tclu_log_message "Starting Pro-Installer Version $VPIG(VERSION)"
  vpi.load_user_defaults $VPIG(DEFAULTS_PATH)
  vpi.set_default_resources
  vpi.show_toplevel
  tku_override_tk_getOpenFile 
  
  trace variable VPIG(NAME)         w vpi.name_change_cb
  trace variable VPIG(NAME_VERSION) w vpi.name_version_change_cb
  trace variable VPIG(NAME_OS)      w vpi.name_os_change_cb
  
}

##################################################################
proc vpi.load_user_defaults {PATH} {
   global VPIG
            
   catch {source $PATH}
   
   if { [string length $VPIG(CDROM_PATH)] == 0 } {
      set VPIG(CDROM_PATH) [pwd]
   }
}
##################################################################
proc vpi.save_user_defaults {PATH} {
  global VPIG 
  set VAR_LIST [list \
     SOURCE 		\
     CDROM_PATH 	FTP_SERVER		DAT_DEVICE	DISK_PATH  \
     PRODUCTS_OR_BEANS	\
     NAME 		NAME_VERSION 		NAME_OS \
  ]
    
   if {[catch {open $PATH w} FP]} {
      puts "Defaults cannot be written to $PATH"
      return 
   } 
   
   foreach VAR $VAR_LIST {
      set VALUE $VPIG($VAR)    
      if [string length $VALUE] {
	 set VALUE [tclu_quote_backslashes $VALUE]
	 set COMMAND [format "set VPIG(%s) %s" $VAR $VALUE]
      } else { 
         set COMMAND [format "set VPIG(%s) \"\"" $VAR]
      }
      puts $FP $COMMAND
   }
   if {[catch {close $FP} DUMMY]} {
      puts "Close error. Defaults cannot be written to $PATH"
      return 
   } 
}
   
##################################################################
proc vpi.set_default_resources {} {
  option add *foreground [c_black]
}

##################################################################
proc vpi.show_toplevel {} {
  global VPIG 
  global env
  
  wm title . "Frontline Pro Installer  .... [tclu_get_user]@[info hostname] ($VPIG(OS))" 
  
  set W $VPIG(W)
  frame $W -bg $VPIG(C_BG)
  set WMAIN $VPIG(WMAIN)        
  frame $WMAIN -bg $VPIG(C_BG)
  frame $WMAIN.f_header  -bg $VPIG(C_BG) -relief groove -bd 2
  frame $WMAIN.f_source  -bg $VPIG(C_BG) -relief sunken -bd 2 
  frame $WMAIN.f_name    -bg $VPIG(C_BG) -relief sunken -bd 2
  frame $WMAIN.f_action  -bg $VPIG(C_BG) -relief sunken -bd 2 
  frame $WMAIN.f_control -bg $VPIG(C_BG) -relief groove -bd 2 
  pack  $WMAIN.f_header $WMAIN.f_source $WMAIN.f_name \
        $WMAIN.f_action $WMAIN.f_control \
        -expand true -fill x

  vpi.show_f_header $WMAIN.f_header

  vpi.show_f_source $WMAIN.f_source

  vpi.show_f_name   $WMAIN.f_name

  vpi.show_f_action $WMAIN.f_action

  vpi.show_f_control $WMAIN.f_control
     
  frame $VPIG(WPLUGIN) -relief sunken -bg [c_dblue] -bd 4
  label $VPIG(WPLUGIN_EMPTY) -text "CAMѵ\n \n רҵѵ·CAMʦӭ־ѯ\n Genesis̼QQȺһ:215621679Ⱥ:43023812 \n Ⱥ21641195 ϵQQ496215988\n ϵ绰13410961621  \n һɸıˣĥǳɹһ룡 \n \n \n \n 

========================================
ѧϰо֮ãҵ;
ʹ24Сʱɾϵڰ±˾ \n 
" -width 51 \
                      -font [tbi18] -bg [c_dblue]
  pack $VPIG(WPLUGIN_EMPTY) -fill both -expand 1
     
  pack $WMAIN $VPIG(WPLUGIN) -side left -fill y -expand 1
  label $VPIG(WMESSAGE) -bg [c_yellow] -font [tbr14] \
                        -relief groove -bd 3 -text " \n  "
  pack $W $VPIG(WMESSAGE) -side top -fill x -expand 1
  tku_long_command_set_wmessage $VPIG(WMESSAGE)
}

##################################################################
proc vpi.show_f_header {W} {
  global VPIG 
  
  set WMAIN $VPIG(WMAIN)

  image create photo frontline_gif -file [file join install frontline.gif]
  label $W.l_main -image frontline_gif -bg $VPIG(C_BG) -bd 1 

  set LVERSION "ǰװ汾Ϊ- Genesis $VPIG(VERSION)"
  label $W.l_version -text $LVERSION -bg $VPIG(C_BG) \
        -font [tmi18]

  set WS $W.f_source
  frame $WS -bg $VPIG(C_BG) -relief sunken -bd 2
  label $WS.l_source -text "װԴ" -bg $VPIG(C_BG)  -bd 3 \
        -font [tmr18] -relief groove 
  pack $WS.l_source -pady 3
  
  frame $WS.f_radio -bg $VPIG(C_BG)
#  
  frame $WS.f_radio1 -bg $VPIG(C_BG)
  frame $WS.f_radio2 -bg $VPIG(C_BG)
  frame $WS.f_radio2.2l -bg $VPIG(C_BG)
  frame $WS.f_radio2.2l.t -bg $VPIG(C_BG)
  frame $WS.f_radio2.2l.b -bg $VPIG(C_BG)
  frame $WS.f_radio2.2r -bg $VPIG(C_BG)
#  
  radiobutton $WS.f_radio1.rb -text CDROM \
                -value CDROM \
                -variable VPIG(SOURCE) \
                -highlightbackground [c_dblue] \
                -bg $VPIG(C_BG) -font [tku_font tbr12 u] \
                -command [list cb_source $WMAIN]
  canvas $WS.f_radio1.a -width 7.7c -height 1c -bg $VPIG(C_BG) \
         -highlightbackground [c_dblue]
  $WS.f_radio1.a create line 0c 0.5c 2.6c 0.5c -width 3
  $WS.f_radio1.a create text 2.7c 0.5c -font [tbi14] -anchor w \
                 -text "װ"   
  $WS.f_radio1.a create line 4.2c 0.5c 7.7c 0.5c -arrow last -width 3
  pack $WS.f_radio1.rb $WS.f_radio1.a  -side left
#  
  radiobutton $WS.f_radio2.2l.t.rb -text FTP \
                -value FTP \
                -variable VPIG(SOURCE) \
                -highlightbackground [c_dblue] \
                -bg $VPIG(C_BG) -font [tku_font tbr12 u] \
                -command [list cb_source $WMAIN]
  pack $WS.f_radio2.2l.t.rb -side left 
#  
  radiobutton $WS.f_radio2.2l.b.rb -text DAT \
                -value DAT \
                -variable VPIG(SOURCE) \
                -highlightbackground [c_dblue] \
                -bg $VPIG(C_BG) -font [tku_font tbr12 u] \
                -command [list cb_source $WMAIN]
  pack $WS.f_radio2.2l.b.rb -side left 
#
  canvas $WS.f_radio2.2r.c1 -width 3.5c -height 2c -bg $VPIG(C_BG) \
         -highlightbackground [c_dblue]
  $WS.f_radio2.2r.c1 create line 0c 0.5c 0.5c 0.5c -width 3 -cap round	 
  $WS.f_radio2.2r.c1 create line 0c 1.5c 0.5c 1.5c -width 3 -cap round	 
  $WS.f_radio2.2r.c1 create line 0.5c 0.5c 0.5c 1.5c -width 3 -cap round	 
  $WS.f_radio2.2r.c1 create line 0.5c 1c 1c 1c -width 3	 
  $WS.f_radio2.2r.c1 create text 1.1c 1c -font [tbi14] -anchor w \
                 -text "Retrieve"   
  $WS.f_radio2.2r.c1 create line 3.0c 1c 3.5c 1c -width 3 -arrow last	 
  radiobutton $WS.f_radio2.2r.rb -text DISK \
                -value DISK \
                -variable VPIG(SOURCE) \
                -highlightbackground [c_dblue] \
                -bg $VPIG(C_BG) -font [tku_font tbr12 u] \
                -command [list cb_source $WMAIN]
  canvas $WS.f_radio2.2r.c2 -width 3c -height 2c -bg $VPIG(C_BG) \
         -highlightbackground [c_dblue]
  $WS.f_radio2.2r.c2 create line 0c 1c 0.7c 1c -width 3
  $WS.f_radio2.2r.c2 create text 0.8c 1c -font [tbi14] -anchor w \
                 -text "װ"   
  $WS.f_radio2.2r.c2 create line 2.3c 1c 3c 1c -width 3 -arrow last	 
  pack  $WS.f_radio2.2r.c1 $WS.f_radio2.2r.rb $WS.f_radio2.2r.c2 -side left 
#
  pack $WS.f_radio2.2l.t $WS.f_radio2.2l.b  -side top 
  pack $WS.f_radio2.2l $WS.f_radio2.2r -side left 
#  
  pack $WS.f_radio1 $WS.f_radio2 -side top 
  pack $WS.f_radio
  pack $W.l_main $W.l_version $WS -fill x
}
#................................................................#
proc cb_source {W} {
  vpi.show_f_source $W.f_source
  vpi.show_f_action $W.f_action
}

##################################################################
proc vpi.show_f_source {W} {
  global VPIG 
 
  set CHILDREN [winfo children $W]
  foreach CHILD $CHILDREN {
    if { [string length $CHILD] > 0 } {
      destroy $CHILD
    }
  }

  switch -- $VPIG(SOURCE) {
    CDROM { vpi.show_f_source_cdrom $W }
    FTP   { vpi.show_f_source_ftp   $W }
    DAT   { vpi.show_f_source_dat   $W }
    DISK  { vpi.show_f_source_disk  $W }
    default { bgerror "Switch error install.tcl (0)"}
  }
}

#................................................................#
proc vpi.show_f_source_cdrom {W} {
  global VPIG 
  set WF $W.f_source
  frame $WF -bg $VPIG(C_BG)
  pack $WF
     
  set DIRTYPES [list \
        Products 	products \
	Beans   	beans	\
	]
  frame $WF.cdrom  -bg $VPIG(C_BG)
  button $WF.cdrom.b -text "ԭļ·:"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] \
     -command vpi.cb_cdrom_path
  entry  $WF.cdrom.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(CDROM_PATH) -width 30
  pack $WF.cdrom.b $WF.cdrom.e -side left -fill x  
  frame $WF.type  -bg $VPIG(C_BG)
  set I 0
  foreach {DTTEXT DT} $DIRTYPES {
    radiobutton $WF.type.rb$I -text $DTTEXT \
                -variable VPIG(PRODUCTS_OR_BEANS) \
                -highlightbackground [c_dblue] \
                -value $DT \
                -bg $VPIG(C_BG) -font [tku_font tbr12]
    pack $WF.type.rb$I -side left -fill x -padx 0
    incr I
  }       
  button $WF.type.list -text "List..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.list_contents_cb
  pack $WF.type.list -side right
  pack $WF.cdrom $WF.type -fill x	   
}

proc vpi.cb_cdrom_path {} {
  global VPIG 
  
   set PATH [tk_getOpenFile -initialdir \
             [tclu_get_dir_for_file $VPIG(CDROM_PATH)]]
   if { [string length $PATH] > 0 } { 
      set VPIG(CDROM_PATH) $PATH
   }
}

#................................................................#
proc vpi.show_f_source_ftp {W} {
  global VPIG 
  set WF $W.f_source
  frame $WF -bg $VPIG(C_BG)
  pack $WF
     
  set DIRTYPES [list \
        Products 	products \
	Beans   	beans	\
	]
  frame $WF.ftp  -bg $VPIG(C_BG)
  button $WF.ftp.b -text "FTP Server :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] \
     -command vpi.cb_ftp_server
  entry  $WF.ftp.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(FTP_SERVER)  -width 30
  pack $WF.ftp.b $WF.ftp.e -side left -fill x  
#     
  frame $WF.disk  -bg $VPIG(C_BG)
  button $WF.disk.b -text "· :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14]  \
     -command vpi.cb_disk_path
  entry  $WF.disk.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(DISK_PATH)  -width 30
  pack $WF.disk.b $WF.disk.e -side left -fill x  
#
  frame $WF.type  -bg $VPIG(C_BG)
  set I 0
  foreach {DTTEXT DT} $DIRTYPES {
    radiobutton $WF.type.rb$I -text $DTTEXT \
                -variable VPIG(PRODUCTS_OR_BEANS) \
                -highlightbackground [c_dblue] \
                -value $DT \
                -bg $VPIG(C_BG) -font [tku_font tbr12]
    pack $WF.type.rb$I -side left -fill x -padx 0
    incr I
  }       
  button $WF.type.list -text "List..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.list_contents_ftp_cb
  pack $WF.type.list -side right
  pack $WF.ftp $WF.disk $WF.type -fill x	   
}

#................................................................#
proc vpi.cb_ftp_server {} {
  global VPIG 
  
  set IND 0
  foreach {SERVER TITLE DIR} $VPIG(FTP_LOCATIONS) {  
     set FTP_SERVERS($IND) \
       [format "%-22s----> %s" $SERVER $TITLE]
     incr IND
  }
  vpi.popup_list FTP_SERVERS 3
}

#................................................................#
proc vpi.show_f_source_dat {W} {
  global VPIG 
  set WF $W.f_source
  frame $WF -bg $VPIG(C_BG)
  pack $WF
     
  frame $WF.dat  -bg $VPIG(C_BG)
  button $WF.dat.b -text "DAT Device :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] \
     -command vpi.cb_dat_device
  entry  $WF.dat.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(DAT_DEVICE)  -width 30
  pack $WF.dat.b $WF.dat.e -side left -fill x  
#     
  frame $WF.disk  -bg $VPIG(C_BG)
  button $WF.disk.b -text "· :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] \
     -command vpi.cb_disk_path
  entry  $WF.disk.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(DISK_PATH)  -width 30
  pack $WF.disk.b $WF.disk.e -side left -fill x  
#
  frame $WF.type  -bg $VPIG(C_BG)
  button $WF.type.list -text "List..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.list_contents_dat_cb
  pack $WF.type.list -side right
  pack $WF.dat $WF.disk $WF.type -fill x	   
}

#................................................................#
proc vpi.cb_dat_device {} {
  global VPIG 
  
  set DEVICE_LIST [glob "/dev/rmt/*"]
  set J 0
  foreach DEVICE $DEVICE_LIST {
    set DAT_DEVICES($J) $DEVICE
    incr J
  }
  vpi.popup_list DAT_DEVICES 4
}


#................................................................#
proc vpi.show_f_source_disk {W} {
  global VPIG
  set WF $W.f_source
  frame $WF -bg $VPIG(C_BG)
  pack $WF
     
  set DIRTYPES [list \
	Products 	products \
	Beans   	beans \
	]
  frame $WF.disk  -bg $VPIG(C_BG)
  button $WF.disk.b -text "· :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] \
     -command vpi.cb_disk_path
  entry  $WF.disk.e -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(DISK_PATH)  -width 30
  pack $WF.disk.b $WF.disk.e -side left -fill x  
  frame $WF.type  -bg $VPIG(C_BG)
  set I 0
  foreach {DTTEXT DT} $DIRTYPES {
    radiobutton $WF.type.rb$I -text $DTTEXT \
                -variable VPIG(PRODUCTS_OR_BEANS) \
                -value $DT \
                -highlightbackground [c_dblue] \
                -bg $VPIG(C_BG) -font [tku_font tbr12] \
                -command [list vpi.show_f_action $VPIG(WMAIN).f_action]
    pack $WF.type.rb$I -side left -fill x -padx 0
    incr I
  }       
  button $WF.type.list -text "List..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.list_contents_cb
  pack $WF.type.list -side right
  pack $WF.disk $WF.type -fill x	   
}

proc vpi.cb_disk_path {} {
  global VPIG 
  
   set INITIAL_DIR $VPIG(DISK_PATH)
   set VPIG(DISK_PATH) [string trim $VPIG(DISK_PATH)]
   if { $VPIG(DISK_PATH) == "" } {
      set INITIAL_DIR [file join [tclu_get_tmp_dir]]
   }
   set PATH [tk_getOpenFile \
             -initialdir [tclu_get_dir_for_file $INITIAL_DIR]]
   if { [string length $PATH] > 0 } { 
      set VPIG(DISK_PATH) $PATH
   }
}

##################################################################
proc vpi.list_contents_cb {} {
  global VPIG 
   
  set PATH [vpi.get_dir]
  set NAME_TITLE [vpi.get_dir_type]
     
  if { ! [tclu_is_path_dir $PATH] } {
     tku_popup_message warning ok [format "Illegal directory : \n%s" $PATH]
     return
  }
  
  set TMP_PATH [tclu_get_tmp_name "vpi_list"]
  if {[catch {open $TMP_PATH w} FP]} {
     tku_popup_message error ok [format "Cannot open file\n%s" $TMP_PATH]
     return
  }
  puts $FP [format "()title List of %s in %s\n" $NAME_TITLE $PATH]
  if [catch { glob [file join $PATH *]} NAME_LIST ] {
     set NAME_LIST ""     
  }
  foreach  PATH_NAME $NAME_LIST {
    set NAME [file tail $PATH_NAME]
    set PATH_TITLE [file join $PATH_NAME ${NAME}.ttl]
  
    set TITLE "<No Title>"
    if {! [catch {open $PATH_TITLE r} TFP]} {
      gets $TFP TITLE
      close $TFP
    }
    puts $FP [format "%-15s --> %s" $NAME $TITLE] 
    if [catch { glob [file join $PATH_NAME *]} VERSIONS_LIST ] {
       set VERSIONS_LIST ""     
    }
    foreach PATH_VERSION $VERSIONS_LIST {
      set NAME_VERSION [file tail $PATH_VERSION]
      if { [tclu_str_cmp_nc $NAME_VERSION ${NAME}.ttl] == 0 } continue
      if [catch { glob [file join $PATH_VERSION *]} OSS_LIST ] {
         set OSS_LIST ""     
      }
      if {$OSS_LIST != ""} {
         puts -nonewline $FP [format "    Version %s   for operating systems: " \
		$NAME_VERSION]
	 foreach NAME_OS $OSS_LIST {
	    set NAME_OS [file tail $NAME_OS]
	    puts -nonewline $FP "$NAME_OS"
	    puts -nonewline $FP " "
	 } 
      } else {
            puts $FP \
	       [format "()error     Version %s   No operating system available" \
	 	       $NAME_VERSION]		
      }
      puts $FP " "
    }
    if {$VERSIONS_LIST == ""} {
       puts $FP "()error No versions available"
    }
    puts $FP "()underline                                                      "
  }
  close $FP
	 tku_popup_file $TMP_PATH start "-width 100"
  file delete $TMP_PATH
   
}

##################################################################
proc vpi.cb_name_dat {W} {
  global VPIG 
  
  set STATUS [tclu_get_dat_lst_file $VPIG(DAT_DEVICE) DAT_LIST]
  if { $STATUS != "ok" } {
    puts "while extracting products list from DAT, status is $STATUS"
    tku_popup_message warning ok "Error extracting DAT's products list"
    return
  }
  
  catch {unset NAME}
  set I 0
  foreach {PROD VER OS TITLE} $DAT_LIST {
    if { $I > 0 } {
      set PREV_IND [incr I -1]
      set PREV $NAMES($PREV_IND)
      if { [string compare $PROD $PREV] != 0 } {
	set NAMES($I) $PROD
      }
    } else {
      set NAMES($I) $PROD
    }
    incr I    

  }
  vpi.popup_list NAMES 0
  return  
}


##################################################################
proc vpi.list_contents_ftp_cb {} {
  global VPIG 

  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } return
   
  set TYPE $VPIG(PRODUCTS_OR_BEANS) 

  set LST_PATH "$SOURCE/.$TYPE.lst"
  set DEST_PATH [tclu_get_tmp_name "frontline_install_list"]
  
  set STATUS [vpi.ftp_command DUMMY get $LST_PATH 0 $DEST_PATH]
  if {$STATUS != 0} {
     return
  }
  
  tku_popup_file $DEST_PATH start "-width 50"
  tclu_del_tmp_files  
}
 

##################################################################
proc vpi.show_f_name {W} {
   global VPIG 

   set WF1 $W.f_name
   frame $WF1 -bg $VPIG(C_BG)
   button $WF1.b_name -text "ڰװ:"  \
          -bg [c_yellow] -font [cbr14]      \
          -highlightbackground [c_dblue] \
          -command [list vpi.cb_name $W]
   entry  $WF1.e_name  -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(NAME)
   pack $WF1.b_name $WF1.e_name -side left 

   set WF2 $W.f_version
   frame $WF2 -bg $VPIG(C_BG)
   button $WF2.b_version -text "۳汾:" \
          -bg [c_yellow] -font [cbr14]     \
          -highlightbackground [c_dblue] \
          -command [list vpi.cb_name_version $W]
   entry  $WF2.e_version  -bg [c_yellow] -font [cbr14]  \
         -textvariable VPIG(NAME_VERSION)
   pack $WF2.b_version $WF2.e_version -side left 

   set WF3 $W.f_os
   frame $WF3 -bg $VPIG(C_BG)
   button $WF3.b_os -text "ܲϵͳ:"  \
          -bg [c_yellow] -font [cbr14] \
          -highlightbackground [c_dblue] \
          -command [list vpi.cb_name_os $W]
   entry  $WF3.e_os  -bg [c_yellow] -font [cbr14] \
         -textvariable VPIG(NAME_OS)
   pack $WF3.b_os $WF3.e_os -side left 
   
   pack  $WF1 $WF2 $WF3 -fill x
}

##################################################################
proc vpi.cb_name {W} {
  global VPIG 
  
  switch -- $VPIG(SOURCE) {
    CDROM { vpi.cb_name_cdrom_disk $W }
    FTP   { vpi.cb_name_ftp $W }
    DAT   { vpi.cb_name_dat $W }
    DISK  { vpi.cb_name_cdrom_disk $W }
    default { bgerror "Switch error install.tcl (1)"}
  }
}

#................................................................#
proc vpi.cb_name_cdrom_disk {W} {
  global VPIG 
  
  set PATH [vpi.get_dir]
  if { ! [tclu_is_path_dir $PATH] } {
     tku_popup_message warning ok [format "Illegal directory : \n%s" $PATH]
     return
  }
  if [catch { glob [file join $PATH *]} NAME_LIST ] {
      set NAME_LIST ""     
  }
  catch {unset NAMES}
  set I 0
  foreach NAME_PATH $NAME_LIST {
    set NAME [file tail $NAME_PATH]
    set PATH_TITLE [file join $NAME_PATH ${NAME}.ttl]
    
    set TITLE "<No Title>"
    if {! [catch {open $PATH_TITLE r} FP]} {
      gets $FP TITLE
      close $FP
    }
    set NAMES($I) [format "%-15s --> %s" $NAME $TITLE]
    incr I
  }
  vpi.popup_list NAMES 0
}

proc vpi.cb_name_ftp {W} {
  global VPIG
  set STATUS 0
  
  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } return
  set TYPE $VPIG(PRODUCTS_OR_BEANS) 

  set SOURCE_DIR "$SOURCE/$TYPE"
  set STATUS [vpi.ftp_command FILES_LIST list $SOURCE_DIR]
  if {$STATUS != 0} {
     return
  }
  catch {unset NAMES}
  set IND 0
  foreach FILE $FILES_LIST {
    set SIZE [string length $FILE]    
    if {$SIZE > 20} {
      set COUNT [scan $FILE "%s%s%s%s%s%s%s%s%s" SEC HLINK OWNER GGOUP \
    	         SIZE MONTH DATE TIME NAME]
      if { $COUNT < 9 } continue 
      if [tclu_is_path_dot $NAME] continue
      if [tclu_is_path_hidden $NAME] continue
      
      set TITLE "<No Title>"
      set SOURCE_TITLE "$SOURCE_DIR/$NAME/$NAME.ttl"
      set DEST_TITLE [tclu_get_tmp_name tmp_ttl]
      set STATUS [vpi.ftp_command DUMMY get $SOURCE_TITLE 0 $DEST_TITLE]
      if {$STATUS != 0} {
         return
      }
      if {! [catch {open $DEST_TITLE r} FP]} {
        gets $FP TITLE
        close $FP
	file delete $DEST_TITLE
      }
      set NAMES($IND) [format "%-15s --> %s" $NAME $TITLE]
      incr IND
    }
  }
  vpi.popup_list NAMES 0
}

proc vpi.list_contents_dat_cb {} {
  global VPIG 

  set STATUS [tclu_get_dat_lst_file $VPIG(DAT_DEVICE) DAT_LIST]
  if { $STATUS != "ok" } {
    puts "while extracting products list from DAT, status is $STATUS"
    tku_popup_message warning ok "Error extracting DAT's products list"
    return
  }

  set TMP_PATH [tclu_get_tmp_name "vpi_list"]
  if {[catch {open $TMP_PATH w} FP]} {
     tku_popup_message error ok [format "Cannot open file\n%s" $TMP_PATH]
     return
  }

  puts $FP [format "()title List of %s available on DAT" $VPIG(PRODUCTS_OR_BEANS)]
	    

  foreach {PROD VER OS TITLE} $DAT_LIST {
    
    puts $FP " "
    puts $FP [format "%-15s --> %s" $PROD $TITLE] 
    puts -nonewline $FP [format "    Version %s   for operating system: " $VER]
    puts $FP "$OS "
    puts $FP "()underline                                                      "    
      
  }
  
  close $FP
	 tku_popup_file $TMP_PATH start "-width 100"
  file delete $TMP_PATH
   
  return 
}

##################################################################
proc vpi.cb_name_version {W} {
  global VPIG 
  
  switch -- $VPIG(SOURCE) {
    CDROM { vpi.cb_name_version_cdrom_disk $W }
    FTP   { vpi.cb_name_version_ftp $W }
    DAT   { vpi.cb_name_version_dat $W }
    DISK  { vpi.cb_name_version_cdrom_disk $W }
    default { bgerror "Switch error install.tcl (2)"}
  }
}

#................................................................#
proc vpi.cb_name_version_cdrom_disk {W} {
  global VPIG 

  if { [string length [string trim $VPIG(NAME)]] == 0 } {
     tku_popup_message warning ok "Name is not defined"
     return
  }	
     
  set PATH [file join [vpi.get_dir] $VPIG(NAME)]
  if { ! [tclu_is_path_dir $PATH] } {
     tku_popup_message warning ok [format "Illegal directory : \n%s" $PATH]
     return
  }
  if [catch { glob [file join $PATH *]} VERSIONS_LIST ] {
      set VERSIONS_LIST ""     
  }
  catch {unset VERSIONS}
  set J 0
  foreach NAME_VERSION $VERSIONS_LIST {
    set NAME_VERSION [file tail $NAME_VERSION]
    if {! [tclu_str_cmp_nc $NAME_VERSION $VPIG(NAME).ttl] } continue 
    set VERSIONS($J) $NAME_VERSION
    incr J
  }
  vpi.popup_list VERSIONS 1
}

#................................................................#
proc vpi.cb_name_version_ftp {W} {
  global VPIG
  
  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } return
  set TYPE   $VPIG(PRODUCTS_OR_BEANS) 
  set NAME   $VPIG(NAME)
  
  if { [string length [string trim $VPIG(NAME)]] == 0 } {
     tku_popup_message warning ok "Name is not defined"
     return
  }	
  
  set SOURCE_DIR "$SOURCE/$TYPE/$NAME"
  set STATUS [vpi.ftp_command VER_LIST list $SOURCE_DIR]
  if {$STATUS != 0} {
     return
  }
  catch {unset VERSIONS}
  set IND 0
  foreach VER $VER_LIST {
    set SIZE [string length $VER]    
    if {$SIZE > 20} {
      set COUNT [scan $VER "%s%s%s%s%s%s%s%s%s" SEC HLINK OWNER GGOUP \
	  SIZE MONTH DATE TIME NAME_VERSION]
      if { $COUNT < 9 } { continue } 
      if [tclu_is_path_dot $NAME_VERSION] continue
      if {! [tclu_str_cmp_nc $NAME_VERSION $VPIG(NAME).ttl] } continue 
      set VERSIONS($IND) $NAME_VERSION
      incr IND
    }
  }
  vpi.popup_list VERSIONS 1
}

#................................................................#
proc vpi.cb_name_version_dat {W} {
  global VPIG 
  
  set STATUS [tclu_get_dat_lst_file $VPIG(DAT_DEVICE) DAT_LIST]
  if { $STATUS != "ok" } {
    puts "while extracting products list from DAT, status is $STATUS"
    tku_popup_message warning ok "Error extracting DAT's products list"
    return
  }
  
  catch {unset NAME}
  set I 0
  foreach {PROD VER OS TITLE} $DAT_LIST {
    if { [string compare $PROD $VPIG(NAME)] != 0} continue
    if { $I > 0 } {
      set PREV_IND [incr I -1]
      set PREV $NAMES($PREV_IND)
      if { [string compare $VER $PREV] != 0 } {
	set NAMES($I) $VER
      }
    } else {
      set NAMES($I) $VER
    }
    incr I    
  }
  vpi.popup_list NAMES 1
  return 
}

##################################################################
proc vpi.cb_name_os {W} {
  global VPIG 
  
  switch -- $VPIG(SOURCE) {
    CDROM { vpi.cb_name_os_cdrom_disk $W }
    FTP   { vpi.cb_name_os_ftp $W }
    DAT   { vpi.cb_name_os_dat $W }
    DISK  { vpi.cb_name_os_cdrom_disk $W }
    default { bgerror "Switch error install.tcl (3)"}
  }
}

#................................................................#
proc vpi.cb_name_os_cdrom_disk {W} {
  global VPIG 
  
  if { [string length [string trim $VPIG(NAME)]] == 0 } {
     tku_popup_message warning ok "Name is not defined"
     return
  }	
     
  if { [string length [string trim $VPIG(NAME_VERSION)]] == 0 } {
     tku_popup_message warning ok "Version name is not defined"
     return
  }	
     
  set PATH \
    [file join [vpi.get_dir] $VPIG(NAME) $VPIG(NAME_VERSION)]
  if { ! [tclu_is_path_dir $PATH] } {
     tku_popup_message warning ok [format "Illegal directory : \n%s" $PATH]
     return
  }
  if [catch { glob [file join $PATH *]} OSS_LIST ] {
      set OSS_LIST ""     
  }
  catch {unset OSS}
  set J 0
  foreach NAME_OS $OSS_LIST {
    set NAME_OS [file tail $NAME_OS]
    set OSS($J) $NAME_OS
    incr J
  }
  vpi.popup_list OSS 2
}

#................................................................#
proc vpi.cb_name_os_ftp {W} {
  global VPIG
  
  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } return
  set TYPE   $VPIG(PRODUCTS_OR_BEANS) 
  set NAME   $VPIG(NAME)
  set VER    $VPIG(NAME_VERSION)
  
  set SOURCE_DIR "$SOURCE/$TYPE/$NAME/$VER"
  set STATUS [vpi.ftp_command OS_LIST list $SOURCE_DIR]
  if {$STATUS != 0} {
     return
  }
  catch {unset OSS}
  set IND 0
  foreach OS $OS_LIST {
    set SIZE [string length $OS]    
    if {$SIZE > 20} {
      set COUNT [scan $OS "%s%s%s%s%s%s%s%s%s" SEC HLINK OWNER GGOUP \
	         SIZE MONTH DATE TIME NAME_OS]
      if { $COUNT < 9 } continue 
      if [tclu_is_path_dot $NAME_OS] continue
      set OSS($IND) $NAME_OS
      incr IND
    }
  }
  vpi.popup_list OSS 2
}

proc vpi.cb_name_os_dat {W} {
  global VPIG 
  
  set STATUS [tclu_get_dat_lst_file $VPIG(DAT_DEVICE) DAT_LIST]
  if { $STATUS != "ok" } {
    puts "while extracting products list from DAT, status is $STATUS"
    tku_popup_message warning ok "Error extracting DAT's products list"
    return
  }
  
  catch {unset NAME}
  set I 0
  foreach {PROD VER OS TITLE} $DAT_LIST {
    if { [string compare $PROD $VPIG(NAME)] != 0} continue
    if { [string compare $VER $VPIG(NAME_VERSION)] != 0} continue
    set NAMES($I) $OS
    incr I    
  }
  vpi.popup_list NAMES 2
  return 
  
}

#
# The next 3 procedures are called by the trace mechanism. Due
# to some unexplained behaviour on Solaris, we use 'catch' to avoid
# any unpleasantries
##################################################################
proc vpi.name_change_cb { NAME1 NAME2 OP } {
  global VPIG 
  catch { set VPIG(NAME_VERSION) ""  }
  catch { set VPIG(NAME_OS) ""  }
  catch { vpi.unload }
}

##################################################################
proc vpi.name_version_change_cb { NAME1 NAME2 OP } {
  global VPIG 
  catch { set VPIG(NAME_OS) ""  }
  catch { vpi.unload }
}

##################################################################
proc vpi.name_os_change_cb { NAME1 NAME2 OP } {
  global VPIG 
  catch { vpi.unload }
}

proc vpi.unload { } {
  global VPIG 
   if { $VPIG(INSTALL_STATE) == 1 } {
     vpi.install_cb  $VPIG(WMAIN).f_action.f_buttons.install
   }
}
##################################################################
proc vpi.show_f_action {W} {
  global VPIG 
  
 set CHILDREN [list [winfo children $W]]
 foreach CHILD $CHILDREN {
   if { [string length $CHILD] > 0 } {
     destroy $CHILD
   }
 }

 switch -- $VPIG(SOURCE) {
   CDROM { vpi.show_f_action_cdrom $W }
   FTP   { vpi.show_f_action_ftp   $W }
   DAT   { vpi.show_f_action_dat   $W }
   DISK  { vpi.show_f_action_disk  $W }
   default { bgerror "Switch error install.tcl (4)"}
 }
}

#................................................................#
proc vpi.show_f_action_cdrom {W} {
  global VPIG 
  set WF $W.f_buttons
  frame $WF -bg $VPIG(C_BG)
  pack $WF -fill x

  button $WF.info  -text "Ϣ..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.info_cb
  button $WF.install       -text "ݶȡװϢ" -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command [list vpi.install_cb $WF.install]
  pack $WF.info $WF.install -side left -expand 1
}

#................................................................#
proc vpi.show_f_action_ftp {W} {
  global VPIG 
  set WF $W.f_buttons
  frame $WF -bg $VPIG(C_BG)
  pack $WF -fill x

  button $WF.info  -text "Info..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.info_cb
  button $WF.retrieve      -text "ˢ" -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command [list vpi.retrieve_cb]
  pack $WF.info $WF.retrieve -side left -expand 1
}

#................................................................#
proc vpi.show_f_action_dat {W} {
  global VPIG 
  set WF $W.f_buttons
  frame $WF -bg $VPIG(C_BG)
  pack $WF -fill x

  button $WF.info  -text "Info..." -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.info_cb
  button $WF.retrieve      -text "ˢ" -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command [list vpi.retrieve_cb]
  pack $WF.info $WF.retrieve -side left -expand 1
}

#................................................................#
proc vpi.show_f_action_disk {W} {
  global VPIG
  set WF $W.f_buttons
  frame $WF -bg $VPIG(C_BG)
  pack $WF -fill x

  button $WF.info  -text "Info" -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command vpi.info_cb
  button $WF.install -text "ȡװ------" -font [tbr14] -bg [c_aqua] \
                           -highlightbackground [c_dblue] \
                           -command [list vpi.install_cb $WF.install]
  pack $WF.info $WF.install -side left -expand 1
}


##################################################################
proc vpi.info_cb {} {
  global VPIG 

  
   switch -- $VPIG(SOURCE) {
     CDROM { vpi.show_info_cdrom }
     FTP   { vpi.show_info_ftp }
     DAT   { vpi.show_info_dat }       
     DISK  { vpi.show_info_disk }
     default { bgerror "Switch error install.tcl (5)"}
   }
}

#................................................................#
proc vpi.show_info_ftp {} {
  global VPIG 
     
  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } return
  set TYPE   $VPIG(PRODUCTS_OR_BEANS) 
  set NAME   $VPIG(NAME)
  set VER    $VPIG(NAME_VERSION)
  set OS     $VPIG(NAME_OS)
  set FNAME  $VPIG(NAME).doc

  set DOC_PATH "$SOURCE/$TYPE/$NAME/$VER/$OS/$VPIG(NAME).doc"
  set DEST_PATH [tclu_get_tmp_name $NAME.doc]
  
  set STATUS [vpi.ftp_command DUMMY get $DOC_PATH 0 $DEST_PATH]
  if {$STATUS != 0} {
     return
  }
  tku_popup_file $DEST_PATH start
  tclu_del_tmp_files
  
}

#................................................................#
proc vpi.show_info_cdrom {} {
  global VPIG 
  
  set DOC_PATH \
      [file join [vpi.get_dir] \
	  $VPIG(NAME) $VPIG(NAME_VERSION) $VPIG(NAME_OS) $VPIG(NAME).doc]
  tku_popup_file $DOC_PATH start
}

#................................................................#
proc vpi.show_info_dat {} {

  global VPIG 
  
  set CHOICE [tku_popup_message question [list ok cancel] \
		  "The info operation from a DAT Device\
  	        \n might take up to 5 minutes. \
                \n Please confirm ..."]
  
  if { [string compare $CHOICE "cancel"] == 0 } {
    return 
  }
  
  set PROD $VPIG(NAME)
  set VER  $VPIG(NAME_VERSION)
  set OS   $VPIG(NAME_OS)
  set DAT_DEVICE $VPIG(DAT_DEVICE)
  if { [catch { cd [tclu_get_tmp_dir] } RESPONSE ] } {
    tku_popup_message ok error $RESPONSE
    return 
  }

  set INFO_FILE [file join $PROD $VER $OS $PROD.doc]
  set STOP_FILE [format "end_%s_%s_%s" $PROD $VER $OS]
  file delete -force $STOP_FILE
  set TAR $VPIG(TAR)
  set COMMAND "$TAR xf $DAT_DEVICE $INFO_FILE $STOP_FILE"
  set TITLE "Getting info file for $PROD/$VER/$OS"
  set STATUS [tku_long_command $COMMAND $TITLE $VPIG(WMESSAGE) ignore "" $STOP_FILE]		 
  
  if { $STATUS != "ok" } {
    return    
  }  

  set INFO_FILE [file join [tclu_get_tmp_dir] $INFO_FILE]
  if { [file exists $INFO_FILE] } {
    tku_popup_file $INFO_FILE start
    file delete -force [file join [tclu_get_tmp_dir] $PROD]
  } else {
    tku_popup_message warning ok "No info file was founf for $PROD $VER $OS"
  }
    
}

#................................................................#
proc vpi.show_info_disk {} {
  global VPIG 
     
  set DOC_PATH \
     [file join [vpi.get_dir] \
      $VPIG(NAME) $VPIG(NAME_VERSION) $VPIG(NAME_OS) $VPIG(NAME).doc]
     tku_popup_file $DOC_PATH start
}

##################################################################
proc vpi.retrieve_cb {} {
  global VPIG 

   switch -- $VPIG(SOURCE) {
     FTP   { set STATUS [vpi.retrieve_ftp] }
     DAT   { set STATUS [vpi.retrieve_dat] }
     default { bgerror "Switch error install.tcl (6)"}
   }
   tclu_del_tmp_files 
   if { $STATUS == 0 } {
      tku_popup_message info ok \
        "To install the retrieved product/bean,\
       \nplease switch the source to 'DISK' and\
       \nclick on 'ȡװϢ"
   }
}
#................................................................#

proc vpi.retrieve_ftp {} {
  global VPIG 

  set SOURCE [vpi.get_ftp_source_dir]
  if { $SOURCE == "" } {
     return 1
   }
  set TYPE      $VPIG(PRODUCTS_OR_BEANS) 
  set NAME      $VPIG(NAME)
  set VER       $VPIG(NAME_VERSION)
  set OS        $VPIG(NAME_OS)
  set FNAME     $VPIG(NAME).lst
  set SNAME     $VPIG(NAME).sum
  set DISK_PATH $VPIG(DISK_PATH)
  set TGZ_IS_OK 0
  set UP_TO_DATE 1
  set ERROR 0
     
  set STATUS [vpi.check_empty_disk_path $DISK_PATH]
  if { $STATUS != 0 } {
    return  1
  }
  if [tclu_mkdir $DISK_PATH 1] {
     return 1
  }

  set DISK_PATH [file join $DISK_PATH $TYPE]
  if [tclu_mkdir $DISK_PATH 1] {
     return 1
  }

  set DISK_PATH [file join $DISK_PATH $NAME]
  if [tclu_mkdir $DISK_PATH 1] {
     return 1
  }

  set DISK_PATH [file join $DISK_PATH $VER]
  if [tclu_mkdir $DISK_PATH 1] {
     return 1
  }
  set DISK_PATH [file join $DISK_PATH $OS]
  if [tclu_mkdir $DISK_PATH 1] {
     return 1
  }

  set TTL_PATH  "$SOURCE/$TYPE/$NAME/$NAME.ttl"
  set DEST_TTL  [file join $VPIG(DISK_PATH) $TYPE $NAME $NAME.ttl] 
    
  set STATUS [vpi.ftp_command DUMMY get $TTL_PATH 0 $DEST_TTL]
  if {$STATUS < 0} {
    return 1
  }
  # don't check error status, since the ttl file isn't essential
  # for proper installation
  
  set LST_PATH  "$SOURCE/$TYPE/$NAME/$VER/$OS"
  set DEST_PATH [tclu_get_tmp_name $NAME.lst]
  set DEST_SUM  [tclu_get_tmp_name $NAME.sum] 
  
  set STATUS [vpi.ftp_command DUMMY get "$LST_PATH/$SNAME" 0 $DEST_SUM]
  if {$STATUS != 0} {
    return 1
  }
  
  # here, we will see if the data file ("xxx.tgz") exists in the target
  # directory, and if it does, we'll see if its sum matches.
  if {! [catch {open $DEST_SUM r} TFP]} {
    gets $TFP LINE
    close $TFP
    scan $LINE "%s%s%s" FILE_NAME TGZ_SIZE TGZ_SUM 
    set TGZ_FILE [file join $DISK_PATH $FILE_NAME]
    if { [file exists $TGZ_FILE] } {
      set CUR_SUM [tclu_get_checksum $TGZ_FILE]
      set CUR_SIZE [file size $TGZ_FILE]
      if {($CUR_SUM == $TGZ_SUM) && ($CUR_SIZE == $TGZ_SIZE)} {
	set TGZ_IS_OK 1 
      }
    }
  }
  #
  #  Retrieve the .lst file for comparing all current files
  #
  set STATUS [vpi.ftp_command DUMMY get "$LST_PATH/$FNAME" 0 $DEST_PATH]
  if {$STATUS != 0} {
     return 1
  }
  if {! $TGZ_IS_OK} {
    set IGNORE_FILES ""
  } else {
    set IGNORE_FILES "$NAME.tgz.p*"
  }
  set LST_STATUS [tclu_check_lst_file $DISK_PATH $DEST_PATH \
                  $IGNORE_FILES LST_RESULT]
  set FILE_COUNT 0     
  set TPATH [tclu_get_tmp_name "ftp_list"]
  if {[catch {open $TPATH w} FP]} {
    tku_popup_message error ok [format "Cannot open file\n%s" $TPATH]
    return 1
  }
  set NEED_COPY_ANYTHING 0
  puts $FP "               Files On FTP Versus Local Files:\n" 
  puts $FP \
    "File Name-----------Name Match---File Size---Sum Match---Should Copy\n" 
  foreach {FILE_NAME NMATCH FILE_SIZE SUMATCH FILE_SUM COPYF} $LST_RESULT {
    set CUR_CHECK_FILE [file join $DISK_PATH $FILE_NAME]
    set FILE_NAMES($FILE_COUNT) $FILE_NAME
    set FILE_SIZES($FILE_COUNT) $FILE_SIZE
    if { [string compare $COPYF  "yes" ] == 0 } {
      set FILE_COPY($FILE_COUNT) 1
      set NEED_COPY_ANYTHING 1
    } else {
      set FILE_COPY($FILE_COUNT) 0
    }
    set FILE_SUMS($FILE_COUNT)  $FILE_SUM
    
    set OLINE [format "%-20s    %s           %-10d %s           %s" \
		   $FILE_NAME $NMATCH $FILE_SIZE $SUMATCH $COPYF]  
    puts $FP $OLINE 
    incr FILE_COUNT
    if { [string match "*.tgz.p*" $FILE_NAME] } {
      lappend FILES_TO_APPEND [file join $DISK_PATH $FILE_NAME]
    }
  }
  close $FP
  incr FILE_COUNT 
  if { $NEED_COPY_ANYTHING } {
    set CHOICE [tku_popup_message question okcancel \
                "Some files need to be retrieved from FTP.\
  	       \nPlease Check and Confirm:" $TPATH]
    if { [string compare $CHOICE "cancel"] == 0 } {
      tku_popup_message warning ok "Retrieve was stopped by user"
      return  1
    }
  }

  set TPATH [tclu_get_tmp_name "ftp_result_list"]
  if {[catch {open $TPATH w} FP]} {
    tku_popup_message error ok [format "Cannot open file\n%s" $TPATH]
    return 1
  }
  
  set PARTS_RETRIEVED_OK 1
  puts $FP "File Name                 Size               Status\n"
  for {set IND 0} {$IND < $FILE_COUNT-1} {incr IND} {
    if { $LST_STATUS == 0 } break
    if {$FILE_COPY($IND) == 0} continue
    set DEST_PATH [file join $DISK_PATH $FILE_NAMES($IND)]
    set SOURCE_PATH "$LST_PATH/$FILE_NAMES($IND)"
    set STATUS [vpi.ftp_command NAME_LIST get $SOURCE_PATH \
                $FILE_SIZES($IND) $DEST_PATH]
    if {$STATUS < 0} {
      return 1
    }
    set UP_TO_DATE 0
    if { $STATUS != 0 } {
      set OLINE [format "%-20s     Failed to Retrieve" $FILE_NAMES($IND)] 
      set PARTS_RETRIEVED_OK 0
    } else {
      set CUR_SUM [tclu_get_checksum $DEST_PATH]
      if {[string compare $CUR_SUM $FILE_SUMS($IND)] == 0} {
	set OLINE [format "%-20s     %-10d        Retrieved OK" \
		       $FILE_NAMES($IND) $FILE_SIZES($IND)]
	
      } else {
	set OLINE [format "%-20s     %-10d        Checksum Does not match" \
		       $FILE_NAMES($IND) $FILE_SIZES($IND)]
        set PARTS_RETRIEVED_OK 0
      }
    }
    puts $FP $OLINE
  }
  if { ! $PARTS_RETRIEVED_OK } {
    close $FP
    set CHOICE [tku_popup_message error ok \
                "Rerieval errors:" $TPATH] 
    return 1
  } else {
     close $FP
     catch { file delete $TPATH }
  }
  if {! $TGZ_IS_OK} {
    # should take cat from tools directory 
    set APPEND_FILES [file join $DISK_PATH $NAME.tgz.p*]
    set ACT_FILES [lsort $FILES_TO_APPEND]
    set CAT $VPIG(CAT)
    set COMMAND [format "%s %s | %s bs=1000000 of=%s" \
		     $CAT \
		     $ACT_FILES \
		     $VPIG(DD)  \
		     $TGZ_FILE]
    set TITLE "Appending retrieved files ..."
    set STATUS [tku_long_command $COMMAND $TITLE $VPIG(WMESSAGE) \
	 	    ignore ""]
    if { $STATUS != "ok" } {
      tku_popup_message error ok \
         [format "%s/n%s"                     \
         "Failed to append retrieved files."  \
	 "Check log file for errors"]
      return  1
    }
    set UP_TO_DATE 0
  
    set SIZE [file size $TGZ_FILE]
    set SUM  [tclu_get_checksum $TGZ_FILE]
  
    if { ($SIZE == $TGZ_SIZE) && ($SUM == $TGZ_SUM) } {
      set TGZ_IS_OK 1
    }
  }
    
  if {$TGZ_IS_OK} {
    set P_WILD [file join $DISK_PATH $NAME.tgz.p*]
    set P_PATHS [glob -nocomplain $P_WILD]
    foreach P_PATH $P_PATHS {
       catch { file delete $P_PATH } 
    }
  }
  switch -- $TYPE {
     products { set TYPE_NAME Product } 
     beans    { set TYPE_NAME Bean } 
  }
  if { $UP_TO_DATE } {
    tku_popup_message info ok \
      [format "%s %s/%s/%s\nis up to date" \
       $TYPE_NAME $NAME $VER $OS]
  } else {
    tku_popup_message info ok \
      [format "%s %s/%s/%s\nwas retrieved successfully"\
       $TYPE_NAME $NAME $VER $OS]
  }
  return 0
}


#................................................................#
proc vpi.retrieve_dat {} {
  global VPIG
  
  set PROD $VPIG(NAME)
  set VER  $VPIG(NAME_VERSION)
  set OS   $VPIG(NAME_OS)
  set PATH $VPIG(DISK_PATH)
  set TYPE $VPIG(PRODUCTS_OR_BEANS)

  if { [string compare $TYPE "products"] == 0 } {
    set PRINT_TYPE "product"
  } else {
    set PRINT_TYPE "bean"
  }

  set STATUS [vpi.check_empty_disk_path $PATH]
  if { $STATUS != 0 } {
    return  1
  }
  
  if { ! [tclu_is_path_dir $PATH] } {
    if [tclu_mkdir $PATH 1] {
      tku_popup_message error ok "Can't create $PATH, Please check"    
      return 1
    }
  }

  set PATH [file join $PATH $TYPE]
  if [tclu_mkdir $PATH 1] {
    tku_popup_message error ok "Can't create $PATH, Please check"    
    return 1
  }

  
  
  set STOP_NAME [format "end_%s_%s_%s" $PROD $VER $OS] 
  cd $PATH
  set DAT_DEVICE $VPIG(DAT_DEVICE)
  set TAR $VPIG(TAR)
  set COMMAND "$TAR xf $DAT_DEVICE $PROD/$PROD.ttl $PROD/$VER/$OS $STOP_NAME"
  set TITLE "Extracting $PROD from DAT..."
  set STOP_FILE [file join $PATH $STOP_NAME]
  set STATUS [tku_long_command $COMMAND \
		  $TITLE   \
		  $VPIG(WMESSAGE)  \
		  abort \
		  "" \
		  $STOP_FILE]		 
  
  if { $STATUS != "ok" } {
    tku_popup_message error ok "Error retrieving $PROD from DAT"    
    return 1   
  }
  
  if { [file exists [file join $PATH $PROD]] } {
    set CHECK_PATH [file join $PATH $PROD $VER $OS]
    set LST_PATH [file join $CHECK_PATH $PROD.lst]
    set LST_STATUS [tclu_check_lst_file $CHECK_PATH $LST_PATH "*.lst" LST_RESULT 1]

    set TMP_PATH [tclu_get_tmp_name "check_list"]
    if {[catch {open $TMP_PATH w} FP]} {
      tku_popup_message error ok [format "Cannot open file\n%s" $TMP_PATH]
      return 1
    }
    
    set LINE "$PROD Version $VER for $OS check results:"
    puts $FP $LINE
    puts $FP "()underline                                              \n"
    switch -- $LST_STATUS {
      0  {
	tku_popup_message info ok \
	    [format "%s %s/%s/%s\nwas retrieved and checked successfully"\
		 $PRINT_TYPE $PROD $VER $OS]
	return 0
      }
      
      -1 {
	set LINE "Directory $CHECK_PATH does not exist."
	puts $FP $LINE
	set LINE "()error $PRINT_TYPE could not be installed properly\n"
	puts $FP $LINE       
      } 

      -2 {
	set LINE "File $PROD.lst was not found in $CHECK_PATH"
	puts $FP $LINE
	set LINE "()error $PRINT_TYPE could not be installed properly\n"
	puts $FP $LINE       
      }

      -3 {
	set LINE "Failed to open $PROD.lst in $CHECK_PATH"
	puts $FP $LINE
	set LINE "()error $PRINT_TYPE could not be installed properly\n"
	puts $FP $LINE       
      }

      default {	
	foreach {FILE_NAME NMATCH FILE_SIZE SUMATCH FILE_SUM COPYF} $LST_RESULT {
	  if {[string compare $COPYF "no "] == 0} continue
	  if {[string compare $NMATCH "yes"] == 0} {
	    set LINE "checksum for file $FILE_NAME does not match"
	    puts $FP $LINE 
	  } else {
	    set LINE "file $FILE_NAME does not exist"
	    puts $FP $LINE 
	  }
	}
	set LINE "()error $PRINT_TYPE could not be installed properly\n"
	puts $FP $LINE
      }
    }
    close $FP
    tku_popup_file $TMP_PATH start "-width 100"  
    file delete -force $TMP_PATH
    return 1  
  } else {
    set LINE1 "$PRINT_TYPE $PROD/$VER/$OS doesn't exist on DAT \n"
    set LINE2 "Please use \"List\" Button to list available $TYPE"
    tku_popup_message error ok [concat $LINE1 $LINE2] 
    return 1
  }
  
}
##################################################################
proc vpi.check_empty_disk_path { PATH } {
  set PATH [string trim $PATH]
  if { [string length $PATH] == 0 } {
     tku_popup_message error ok \
        "Disk Path must contain a valid path\
       \nto a new or existing directory into which \
       \nthe products/beans will be retrieved."
      return 1
  }
  return 0
}

##################################################################
   
proc vpi.install_cb { WB } {
  global VPIG 

  switch -- $VPIG(INSTALL_STATE) {
     {0} {
      
       set VPIG(PRODUCT_PATH) \
         [file join [vpi.get_dir] \
           $VPIG(NAME) $VPIG(NAME_VERSION) $VPIG(NAME_OS)]
       set INSTALL_PATH \
         [file join $VPIG(PRODUCT_PATH) $VPIG(NAME).ins]
   
       if { ! [tclu_is_path_file $INSTALL_PATH] } {
          tku_popup_message error ok \
  	    [format "No installation file:\n%s" $INSTALL_PATH]
          return
       }
       pack forget $VPIG(WPLUGIN_EMPTY)
       catch {destroy $VPIG(WPLUGIN_FRAME)}
       frame  $VPIG(WPLUGIN_FRAME) -bd 1 -bg [c_dblue]
       pack   $VPIG(WPLUGIN_FRAME) -expand 1 -side top -fill both	  

       tclu_log_message "ȡװϢ"
       tclu_log_message \
	  "  $VPIG(NAME) / $VPIG(NAME_VERSION) / $VPIG(NAME_OS)"	  
##########################	  
vpi.source_plugin $INSTALL_PATH
##########################
       $WB configure -text "ذװԭʼ"       
       set VPIG(INSTALL_STATE) 1
       
     }
     {1} {
	destroy $VPIG(WPLUGIN_FRAME) 
	pack $VPIG(WPLUGIN_EMPTY) -fill both -expand 1 
        $WB configure -text "ȡװϢ"
        set VPIG(INSTALL_STATE) 0
        tclu_log_message "Unloading Installation Plug-in"
        . configure -cursor left_ptr
     }
    default { bgerror "Switch error install.tcl (7)"}
  }
}

#................................................................#
proc vpi.source_plugin {INSTALL_PATH} {
  global VPIG 
     
  source $INSTALL_PATH
}

##################################################################
proc vpi.show_f_control {W} {
  global VPIG 
  set BFONT [tbr14]
  button $W.b_help    \
         -highlightbackground [c_dblue] \
         -text "" \
         -bg [c_aqua]  -bd 1 -font $BFONT \
        -command {vpi.cb_b_help}
  button $W.b_showlog \
        -text "ʾ־" \
         -highlightbackground [c_dblue] \
        -bg [c_aqua] -bd 1 -font $BFONT \
        -command {vpi.cb_b_showlog}
  button $W.b_quit    \
        -text "˳"            \
         -highlightbackground [c_dblue] \
        -bg [c_aqua] -bd 1 -font $BFONT \
        -command {vpi.cb_b_quit}
  pack  $W.b_help              \
        $W.b_showlog           \
        $W.b_quit              \
        -side left -expand 1
}

#................................................................#
proc vpi.cb_b_help {} {
  global VPIG 
   
  set PATH [file join $VPIG(INSTALL_PATH) install.hlp]
  
  tku_popup_file $PATH start "-height 40"
}

#................................................................#
proc vpi.cb_b_showlog {} {
  global VPIG 
   
   tku_popup_file $VPIG(LOG_PATH) end
}

#................................................................#
proc vpi.cb_b_quit {} {
  global VPIG 
  set CHOICE yes

  set CHOICE [tku_popup_message warning yesno "Really Quit ?"]
  if { $CHOICE == "yes" } { 
    tclu_log_message "End Session."  "underline" 
     
    vpi.save_user_defaults $VPIG(DEFAULTS_PATH)
    destroy .  
    exit
  }
}

################ PATH CREATION ROUTINES #######################

proc vpi.get_dir {} {
  global VPIG 
   
   set DIR [string tolower [vpi.get_dir_type]]
   switch -- $VPIG(SOURCE) {
     CDROM { return [file join $VPIG(CDROM_PATH) $DIR] }
     DISK  { return [file join $VPIG(DISK_PATH) $DIR] }
     default { bgerror "Switch error install.tcl (8)" }
   }
}

proc vpi.get_dir_type {} {
  global VPIG 
   
   switch -- $VPIG(SOURCE) {
     CDROM { return $VPIG(PRODUCTS_OR_BEANS) }
     FTP   { return $VPIG(PRODUCTS_OR_BEANS) }
     DAT   { return "products" }
     DISK  { return $VPIG(PRODUCTS_OR_BEANS) }
     default { bgerror "Switch error install.tcl (9)" }
   }
}

################ POPUP LIST #########################

proc vpi.popup_list {UP_OPTIONS FIELD} {
  global VPIG 
  global VPIG_WL
  upvar $UP_OPTIONS   OPTIONS

  set VPIG_WL .w_popup_list
  if { [array size OPTIONS] == 0 } {
    tku_popup_message warning ok \
      "No options are available"
    return
  }
  
  if {[winfo exists $VPIG_WL]} {
    destroy $VPIG_WL
  }

   toplevel $VPIG_WL
   switch -- $FIELD {
     0 {set WNAME "Ŀ¼б"    ; set LWIDTH 100 }
     1 {set WNAME "汾б" ; set LWIDTH 20 }
     2 {set WNAME "Ӧƽ̨" ; set LWIDTH 20 }
     3 {set WNAME "FTP"      ; set LWIDTH 60 }
     4 {set WNAME "װб"      ; set LWIDTH 40 }
     default { bgerror "Switch error install.tcl (11)" }
   }
   wm title $VPIG_WL "$WNAME"
   
   set WFL $VPIG_WL.f_list
   frame $WFL -bg [c_yellow]
   scrollbar $WFL.sb -command "$WFL.list yview"  -bg [c_yellow]
   listbox $WFL.list -yscroll "$WFL.sb set" \
                     -width $LWIDTH  -bg [c_yellow] -font [cbr14]
   pack $WFL.sb   -side right -fill y
   pack $WFL.list -side left  -expand 1 -fill both
   bind $WFL.list <Double-1>  [list vpi.cb_popup_list $FIELD 1 1]

   set WFB $VPIG_WL.f_buttons
   frame $WFB -bg [c_yellow]
   button $WFB.ok     -text OK -bg [c_aqua] \
         -highlightbackground [c_dblue] \
         -command [list vpi.cb_popup_list $FIELD 1 1]
   button $WFB.apply  -text Apply  -bg [c_aqua]\
         -highlightbackground [c_dblue] \
          -command [list vpi.cb_popup_list $FIELD 1 0]
   button $WFB.cancel -text Cancel  -bg [c_aqua]\
         -highlightbackground [c_dblue] \
          -command [list vpi.cb_popup_list $FIELD 0 1]	
   pack $WFB.ok $WFB.apply $WFB.cancel  -side left -expand true

  for {set I 0} {$I < [array size OPTIONS] } {incr I} {
    $WFL.list insert end $OPTIONS($I)
  } 
  pack $WFL $WFB -fill x
  tku_center_on_window . $VPIG_WL 
  wm deiconify $VPIG_WL 
}

#................................................................#
proc vpi.cb_popup_list {FIELD SET CLOSE} {
  global VPIG 

  set NO_SEL [catch {selection get} SEL]
  if {$NO_SEL} {set SET 0} 
  if {$SET} {
    set NAME [lindex $SEL 0]
    switch -- $FIELD {
       0  {set VPIG(NAME)  $NAME}
       1  {set VPIG(NAME_VERSION)  $NAME}
       2  {set VPIG(NAME_OS)       $NAME}
       3  {set VPIG(FTP_SERVER)    $NAME}
       4  {set VPIG(DAT_DEVICE)    $NAME}
       default { bgerror "Switch error install.tcl (12)" }
    }
  }
  if {$CLOSE} {
    wm withdraw .w_popup_list
  }
}

proc vpi.get_ftp_source_dir {} {
  global VPIG 

  set FTP_SERVER [string trim $VPIG(FTP_SERVER)]
  set SOURCE ""

  foreach {SERVER TITLE DIR} $VPIG(FTP_LOCATIONS) {
     if { [string compare $FTP_SERVER $SERVER] == 0 } {
        set SOURCE $DIR 
	break 
     }
  }
  if { $SOURCE == "" } {
      set INDEX [string first "/" $FTP_SERVER]
      if {$INDEX == -1} {
	  tku_popup_message error ok \
	      "Server must have the form of \
              ftp.domain.com/directory\n"
          return
      } else {
	  set LEN [string length $FTP_SERVER]
	  incr LEN -1
	  set VPIG(FTP_SERVER) $FTP_SERVER  
	  set SOURCE [string range $FTP_SERVER $INDEX $LEN]
      }
  }
  return $SOURCE
}


##################################################################
#
# This procedure will perform an FTP command specified by ACTION.
# The command is always executed in background by executing the
# tcl script ftp.tcl.
#
#  UP_RESPONSE   - Will return the response from the ftp.tcl script.
#  ACTION        - list or get
#  SOURCE_PATH   - Directory to list or source file to get
#  EXPECTED_SIZE - The size of the file to get. If non-zero, a progress
#                  in % will be displayed.
#  DEST_PATH     - Only for get command - the desitnation file to create
#
#

proc vpi.ftp_command \
   { UP_RESPONSE ACTION SOURCE_PATH {EXPECTED_SIZE 0} {DEST_PATH ""} } {
  global VPIG
  upvar $UP_RESPONSE RESPONSE
  
  set MESSAGE "FTP:  [string toupper $ACTION] $SOURCE_PATH"
  set RESPONSE_PATH [tclu_get_tmp_name vpi_ftp_response]
  set FTP_LOG_PATH  [tclu_get_tmp_name vpi_ftp_log]
  if { $EXPECTED_SIZE > 0 } {
     set PROGRESS_PATH [tclu_get_tmp_name vpi_ftp_progress]
  } else {
     set PROGRESS_PATH ""
  }

  set INDEX [string first "/" $VPIG(FTP_SERVER)]
  if {$INDEX != -1} {
    incr INDEX -1
    set REAL_SERVER [string range $VPIG(FTP_SERVER) 0 $INDEX] 
  } else {
    set REAL_SERVER $VPIG(FTP_SERVER)
  }

  set COMMAND [format "%s %s %s %s %s %s %s %s %s %s %s" \
		   [info nameofexecutable]  \
		   [file join $VPIG(INSTALL_PATH) ftp.tcl] \
		   $RESPONSE_PATH $FTP_LOG_PATH $REAL_SERVER $ACTION \
		   $SOURCE_PATH   $DEST_PATH    $EXPECTED_SIZE \
		   $PROGRESS_PATH $VPIG(OS) ]
  set STATUS [tku_long_command $COMMAND \
		  $MESSAGE   \
		  $VPIG(WMESSAGE)  \
		  ignore $PROGRESS_PATH ""]
  switch -- $STATUS {
    "ok"      { 
         if [catch {open $RESPONSE_PATH} FP] {
            tku_popup_message error ok \
	          "FTP command failure.\
		 \nPlease press 'show log' for details."
	    catch {file delete $FTP_LOG_PATH}
	    return 1
	 } else {
	    gets $FP RESPONSE
	    close $FP
	    file delete $RESPONSE_PATH
	    if { $RESPONSE == "@ERROR@" } {
              tku_popup_message error ok \
	        [format "%s operation failed for path:\n%s" \
		 $ACTION $SOURCE_PATH] $FTP_LOG_PATH    
	       catch {file delete $FTP_LOG_PATH}
	       return 1
	    } else {
 	       catch {file delete $FTP_LOG_PATH}
	       return 0
	    }
	 }
     }
    "cancel"  { 
         catch {file delete $RESPONSE_PATH}
         catch {file delete $FTP_LOG_PATH}
         return -1 
     }
    "error"   { 
         catch {file delete $RESPONSE_PATH}
         tku_popup_message error ok \
	    [format "%s operation failed for path:\n%s" \
	     $ACTION $SOURCE_PATH] $FTP_LOG_PATH    
	 catch {file delete $FTP_LOG_PATH}
         return 1 
    }
    default { bgerror "Switch error install.tcl (13)" }
  }
  catch {file delete $FTP_LOG_PATH}
}

##################################################################
#
# This procedure fills the paths to useful UNIX commands in the
# global variable VPIG according to the operating system on which
# the script is running on.
#
# The commands are:
#  cat
#  chmod
#  cp
#  dd
#  tar
#
proc vpi.set_unix_commands {} {
global VPIG
set COMMANDS [list CAT CHMOD CP DD TAR WHOAMI]

   switch -- $VPIG(OS) {
     aix {
        set VPIG(CAT)   "/bin/cat"
        set VPIG(CHMOD) "/bin/chmod"
        set VPIG(CP)    "/bin/cp"
        set VPIG(DD)    "/bin/dd"
        set VPIG(TAR)   "/bin/tar"
        set VPIG(WHOAMI) "/bin/whoami"
     } 
     hp10 {
        set VPIG(CAT)   "/bin/cat"
        set VPIG(CHMOD) "/bin/chmod"
        set VPIG(CP)    "/bin/cp"
        set VPIG(DD)    "/bin/dd"
        set VPIG(TAR)   "/bin/tar"
        set VPIG(WHOAMI) "/bin/whoami"
     }
     nt {
        set VPIG(CAT)   [file join $VPIG(TOOLS_PATH) cat.exe]
        set VPIG(CHMOD) [file join $VPIG(TOOLS_PATH) chmod.exe]
        set VPIG(CP)    [file join $VPIG(TOOLS_PATH) cp.exe]
        set VPIG(DD)    [file join $VPIG(TOOLS_PATH) dd.exe]
        set VPIG(TAR)   [file join $VPIG(TOOLS_PATH) tar.exe]
	# No need on NT
        set VPIG(WHOAMI) ""  
     }
     solaris {
        set VPIG(CAT)   "/bin/cat"
        set VPIG(CHMOD) "/bin/chmod"
        set VPIG(CP)    "/bin/cp"
        set VPIG(DD)    "/bin/dd"
        set VPIG(TAR)   "/bin/tar"
        set VPIG(WHOAMI)   "/usr/ucb/whoami"
     }
     linux {
        set VPIG(CAT)   "/bin/cat"
        set VPIG(CHMOD) "/bin/chmod"
        set VPIG(CP)    "/bin/cp"
        set VPIG(DD)    "/bin/dd"
        set VPIG(TAR)   "/bin/tar"
        set VPIG(WHOAMI)   "/usr/bin/whoami"
     }
     default {
        puts "Illegal operating system: $VPIG(OS)"
	exit 1	
     }
   }
   foreach COMMAND $COMMANDS {
        if { $VPIG($COMMAND) == "" } {
	   continue 
	}
        if { ! [file exists $VPIG($COMMAND)] } {
           puts "The command $VPIG($COMMAND) does not exist."
	   puts "Please remedy and rerun."
	   exit 1 
	   }
        if { ! [file executable $VPIG($COMMAND)] } {
           puts "The command $VPIG($COMMAND) is not an executable file."
	   puts "Please remedy and rerun."
	   exit 1 
	}
   }     
   tclu_set_commands \
      $VPIG(CAT) $VPIG(CHMOD) $VPIG(CP) $VPIG(DD) $VPIG(TAR) $VPIG(WHOAMI)   
}

################ PATH CREATION ROUTINES #######################
set  VPIG(TOOLS_PATH) [lindex $argv 0]
set  VPIG(INSTALL_PATH) [file join [pwd] install]
set  VPIG(OS)         [string tolower [file tail $VPIG(TOOLS_PATH)]] 
#if { $VPIG(OS) == "nt" } {
#  console show
#}
puts "װ汾: $VPIG(VERSION)"
source [file join $VPIG(INSTALL_PATH) tclu.tcl]
source [file join $VPIG(INSTALL_PATH) tku.tcl]
source [file join $VPIG(INSTALL_PATH) plugin.tcl]
vpi.main





