#---------------------------------------------------
#
#  Module : reader.tcl
#  Author : MK
#  Date   : 20 Sep 1997
#
#  Description :
#
#  This is the main module for the Frontline Literature Reader.
#
#---------------------------------------------------
#
# Global variables which can be overwritten by the defaults file
#
global READG
global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT NUM_LI
set READG(VERSION)		"1.3"
set READG(PRODUCT)		"Genesis"
set READG(TYPE)			"Articles"
set READG(ACROBAT_COMMAND)      "acroread -useFrontEndProgram"
set READG(CUR_INDEX)            "-1"
set READG(W)                    ".w"   
set READG(W_MESSAGE)		".w.main.wmessage"
set READG(W_LCOUNT)             ""
set READG(W_TITLES)             ""
set READG(W_ABSTRACT)           ""
   
set READG(PRODUCTS) [list \
         General Genesis GenFlex InCAM InPlan InCoupon ]

set READG(TYPES) [list Brochures Overview ]
  	
set READG(PRODUCTS_LC)  [list]
foreach ITEM $READG(PRODUCTS) {
   lappend READG(PRODUCTS_LC) [string tolower $ITEM]
}

set READG(TYPES_LC)  [list]
foreach ITEM $READG(TYPES) {
   lappend READG(TYPES_LC) [string tolower $ITEM]
}

proc reader.main {} {     
  global READG 
  
  tclu_set_tools_dir $READG(TOOLS_PATH)
  tclu_set_os $READG(OS)
  tku_set_tools_dir $READG(TOOLS_PATH)
  tku_set_os $READG(OS)
  reader.set_unix_commands  
  
  set READG(USER_DIR)      [file join [tclu_get_home] .valor_install]
  set READG(DEFAULTS_PATH) [file join $READG(USER_DIR) reader_defaults]  
  set READG(LOG_PATH)      [file join $READG(USER_DIR) reader_log]  
  set READG(C_BG)	   [c_dblue]

  catch {file mkdir $READG(USER_DIR)}

  set READG(COPY_DIR)      [tclu_get_tmp_dir]
  tclu_log_set_path $READG(LOG_PATH)
  tclu_log_newline 
  tclu_log_message "Starting Reader Version $READG(VERSION)"
  reader.load_user_defaults $READG(DEFAULTS_PATH)
  set READG(LIT_PATH) [file join [pwd] literature]
  reader.set_default_resources
  set READG(HEIGHT) [winfo screenheight .]
  if { $READG(HEIGHT) < 768 } {
     puts " Window size is reduced for a small screen " 
  }
  reader.show_toplevel  
  reader.read_index_file
  reader.update_display
  tku_override_tk_getOpenFile 
  tku_cursor_normal       
}

##################################################################
proc reader.load_user_defaults {PATH} {
   global READG
   
   catch {source $PATH}
}

##################################################################
proc reader.save_user_defaults {PATH} {
  global READG 
  set VAR_LIST [list \
       PRODUCT  TYPE  ACROBAT_COMMAND COPY_DIR
  ]
    
   if {[catch {open $PATH w} FP]} {
      puts "Defaults cannot be written to $PATH"
      return 
   } 
   
   foreach VAR $VAR_LIST {
      set VALUE $READG($VAR)    
      if [string length $VALUE] {
	 set COMMAND [format "set READG(%s) \"%s\"" $VAR $VALUE]
      } else { 
         set COMMAND [format "set READG(%s) \"\"" $VAR]
      }
      puts $FP $COMMAND
   }
   close $FP   
}

##################################################################
proc reader.set_default_resources {} {
  option add *foreground [c_black]
  option add *font [cbr14]
}

##################################################################
proc reader.read_index_file {} {
    global READG
    global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT
    global env

    set SHOW_MISSING_FILES 0
    if [info exists env(SHOW_MISSING_FILES)] {
       set SHOW_MISSING_FILES 1
    }
    
    tku_cursor_watch       
    tku_message "Reading index file..."   
    set INDEX_PATH [file join $READG(LIT_PATH) index.txt]
    
    if { [catch {open $INDEX_PATH RDONLY} FP] } {
       tku_popup_message error ok $FP
       return 1
    }
    set INDEX  0
    set LCOUNT 0
    while { ! [eof $FP] } {
       gets $FP LINE
       incr LCOUNT
       if { [string index $LINE 0] != "@" } {
	  continue 
       }
       set COMMAND [string index $LINE 1]
       switch -- $COMMAND {
	  "N" {
	     set NAME [string range $LINE 3 end]
	     set LI_NAME($INDEX) [string trim $NAME]
	  }
	  "T" {
	     set TYPE [string tolower [string range $LINE 3 end]]
	     set LI_TYPE($INDEX) [string trim $TYPE]
	     if { [lsearch $READG(TYPES_LC) $LI_TYPE($INDEX)] < 0 } {
	        puts "Illegal type $LI_TYPE($INDEX) at line $LCOUNT\
		    \n   Document: $LI_NAME($INDEX)"
	     }
	  }
	  "F" {
	     set FILE [string tolower [string range $LINE 3 end]]
	     set LI_FILE($INDEX) [string trim $FILE]	     
             set FPATH [get_li_file_path $INDEX]
	     if { ! [file exists $FPATH] && $SHOW_MISSING_FILES } {
	        puts "Missing: $FPATH"
	     }
	  }
	  "P" {
	     set LI_PRODUCTS($INDEX) \
	         [split [string tolower [string range $LINE 3 end]]]
	     foreach PRODUCT $LI_PRODUCTS($INDEX) {
	        if { [lsearch $READG(PRODUCTS_LC) $PRODUCT] < 0 } {
	           puts "Illegal product $PRODUCT at line $LCOUNT\
	               \n   Document: $LI_NAME($INDEX)"
	        }
	     }
	  }
	  "A" {
 	     reader.add_abstract_to_lit_item $FP $INDEX LCOUNT
	     incr INDEX
	     set LI_NAME($INDEX) ""
	     set LI_TYPE($INDEX) ""
	     set LI_FILE($INDEX) ""
	     set LI_PRODUCTS($INDEX) [list]
             set LI_ABSTRACT($INDEX) ""
	  }
	  default {
	     set MESSAGE \
	       "Illegal code in line $LCOUNT\
	      \nof file $INDEX_PATH."
	      puts $MESSAGE
	  }
       }
    }
    close $FP
    set READG(NUM_TITLES) $INDEX
    tku_message " "   
    return 0
}
##################################################################
proc reader.add_abstract_to_lit_item { FP INDEX LCOUNT_VAR} {
    global READG
    global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT
    upvar $LCOUNT_VAR LCOUNT
    
    set LI_ABSTRACT($INDEX) ""
    while { ! [eof $FP] } {
       gets $FP LINE
       incr LCOUNT
       if { [string index $LINE 0] != "@" } {
	  append LI_ABSTRACT($INDEX) "$LINE\n"
	  continue
       } else {
         break
       }
    }        
}

##################################################################
proc reader.show_toplevel {} {
  global READG 
  global env
  
  wm title . \
    "Frontline Literature Reader .... [tclu_get_user]@[info hostname] ($READG(OS))" 
  
  set W $READG(W)
  frame $W -bg $READG(C_BG)
  set WMAIN $W.main
  frame $WMAIN -bg $READG(C_BG)
  frame $WMAIN.f_header  -bg $READG(C_BG) -relief groove -bd 2
  frame $WMAIN.f_product  -bg $READG(C_BG) -relief sunken -bd 2 
  frame $WMAIN.f_gt   -bg $READG(C_BG) -relief sunken -bd 2
  frame $WMAIN.f_abstract -bg $READG(C_BG) -relief sunken -bd 2
  frame $WMAIN.f_acro_command -bg $READG(C_BG) -relief groove -bd 2
  frame $WMAIN.f_control -bg $READG(C_BG) -relief groove -bd 2 
  pack  $WMAIN.f_control -side bottom -expand true -fill both
  label $READG(W_MESSAGE) -bg [c_yellow] -font [tbr14] \
                        -relief groove -bd 3 -text " \n  "
  pack $READG(W_MESSAGE) -side bottom -fill x -expand 1
  pack $WMAIN.f_acro_command -side bottom -fill x -expand 1
  pack  $WMAIN.f_header $WMAIN.f_product $WMAIN.f_gt $WMAIN.f_abstract \
        -side top -expand true -fill both
  reader.show_f_header $WMAIN.f_header
  reader.show_f_product $WMAIN.f_product
  reader.show_f_gt $WMAIN.f_gt
  reader.show_f_abstract     $WMAIN.f_abstract
  reader.show_f_acro_command $WMAIN.f_acro_command
  reader.show_f_control $WMAIN.f_control
  pack $WMAIN -fill both -expand 1
  pack $W -fill both -expand 1
  tku_long_command_set_wmessage $READG(W_MESSAGE)  
}

##################################################################
proc reader.show_f_header {W} {
  global READG 
  
  image create photo frontline_gif -file [file join install frontline.gif]
  frame $W.l -bg $READG(C_BG)
  label $W.l.main -image frontline_gif -bg $READG(C_BG) -bd 1 
  
  set LVERSION "Literature Reader - Version $READG(VERSION)"
  label $W.l.version -text $LVERSION -bg $READG(C_BG) \
        -font [tmi18]
  pack  $W.l.main $W.l.version -side left -fill x 
  pack $W.l -side top -fill x  
}

##################################################################
proc reader.show_f_product {W} {
  global READG 
     
  frame $W.type  -bg $READG(C_BG)
  set I 0
  foreach PRODUCT $READG(PRODUCTS) {
    set UC_PRODUCT [string toupper $PRODUCT]
    set READG($UC_PRODUCT) 1
    checkbutton $W.type.rb$I -text $PRODUCT \
                -variable READG($UC_PRODUCT) \
                -highlightbackground [c_dblue] \
                -bg $READG(C_BG) -font [tbi18] \
		-command reader.f_product_cb
    pack $W.type.rb$I -side left -fill x -padx 0
    incr I
  }  
  pack $W.type -expand 1 -fill both
}

proc reader.f_product_cb {} {
   reader.update_display
}

##################################################################
proc reader.show_f_gt {W} {
  global READG 

  frame $W.types  -bg $READG(C_BG)
  set I 0
  foreach TYPE $READG(TYPES) {
    radiobutton $W.types.rb$I -text $TYPE \
                -variable READG(TYPE) \
                -highlightbackground [c_dblue] \
                -value $TYPE \
                -bg $READG(C_BG) -font [tku_font tbr12] -height 1 \
		-command reader.f_type_cb
    pack $W.types.rb$I -side top -anchor w -pady 0
    incr I
  }  
  
  set THEIGHT 8
  set TFONT [cbr14]
  if { $READG(HEIGHT) < 768 } {
    set THEIGHT 5 
    set TFONT [tku_font cbr12] 
  }
  frame $W.titles  -bg $READG(C_BG)
  set W_LCOUNT $W.titles.count
  label $W_LCOUNT -bg [c_aqua] -relief groove -bd 2 \
                  -font [cbi14] -text " " 
  set READG(W_LCOUNT) $W_LCOUNT
  set WFL $W.titles.f_list
  set W_TITLES [tku_scrolled_listbox $WFL \
	        "-bg [c_aqua] -height $THEIGHT" \
	        "-bg [c_aqua]"]
  $WFL.list configure -font $TFONT     
  set READG(W_TITLES) $W_TITLES
  pack $W_LCOUNT $WFL -expand 1 -fill both
  bind $W_TITLES <ButtonRelease-1>  [list reader.f_list_cb]
  
  pack $W.types -side left 
  pack $W.titles  -side right 	-expand 1 -fill both
}

proc reader.f_type_cb {} {
   reader.update_display
}

proc reader.f_list_cb {} {
   reader.update_abstract
}

##################################################################
proc reader.show_f_abstract {W} {
  global READG 

  set AHEIGHT 11
  set AFONT [tbr14]
  if { $READG(HEIGHT) < 768 } {
    set AHEIGHT 6 
    set AFONT [tku_font tbr12]
  }
  set WFL $W.label
  if { $READG(HEIGHT) >= 768 } {
    label $WFL -font [tbi18] -bg [c_yellow] -bd 2 -relief groove \
               -text "ABSTRACT" 
    pack $WFL
  }
  set WFT $W.text
  frame $WFT  -bg $READG(C_BG)
  text $WFT.t -yscrollcommand "$WFT.scroll set" \
        -font $AFONT -bg [c_yellow]  -height 11 
  $WFT.t tag configure title \
      -foreground [c_black] -underline 1 -font [tbr18]
  scrollbar $WFT.scroll -command "$WFT.t yview" -bg [c_yellow]
  pack $WFT.scroll -side right -fill y 
  pack $WFT.t -expand 1 -fill both
  set READG(W_ABSTRACT) $WFT.t
  
  pack $WFT    -side top -expand 1 -fill both
}

##################################################################
proc reader.show_f_acro_command {W} {
  global READG 

  set WFACRO $W.acrobat
  frame $WFACRO  -bg $READG(C_BG)
  label $WFACRO.l -text "Acrobat Reader Command :"  \
     -highlightbackground [c_dblue] \
     -bg [c_yellow] -font [cbr14] 
  entry  $WFACRO.e -bg [c_yellow] -font [cbr14] \
         -textvariable READG(ACROBAT_COMMAND) -width 60
  button $WFACRO.b -text "Browse..."  \
     -highlightbackground [c_dblue] \
     -command reader.acro_command_cb \
     -bg [c_yellow] -font [cbr14] 
  pack $WFACRO.b -side right
  pack $WFACRO.l $WFACRO.e -side left -expand 1
       
  pack $WFACRO -side bottom -expand 1 -fill both
}

#####################################################################
proc reader.acro_command_cb {} {
  global READG 
  
  set INITIALDIR "/usr/Acrobat4/bin"
  if { ! [tclu_is_path_dir $INITIALDIR] } {
     set INITIALDIR "/"
  }
  if { $READG(OS) == "nt" } {
     set INITIALDIR "c:/Progra~1/Adobe/Acroba~1.0/Reader"
     if { ! [tclu_is_path_dir $INITIALDIR] } {
        set INITIALDIR "C:/"
     }
  }
  set PATH [tk_getOpenFile -initialdir $INITIALDIR]
  if { [string length $PATH] > 0 } { 
    if { $READG(OS) == "nt" } {
       set READG(ACROBAT_COMMAND) $PATH
    } else {
       set READG(ACROBAT_COMMAND) "$PATH -useFrontEndProgram"
    }
  }
}


##################################################################
proc reader.show_f_control {W} {
  global READG 
  set BFONT [tbr14]
  button $W.b_view    \
         -highlightbackground [c_dblue] \
         -text "View" \
         -bg [c_aqua]  -bd 1 -font [tbr14] \
        -command {reader.cb_b_view}
  button $W.b_copy \
        -text "Copy" \
         -highlightbackground [c_dblue] \
        -bg [c_aqua] -bd 1 -font [tbr14] \
        -command {reader.cb_b_copy}
  button $W.b_help    \
         -highlightbackground [c_dblue] \
         -text "Help" \
         -bg [c_aqua]  -bd 1 -font $BFONT \
        -command {reader.cb_b_help}
  button $W.b_showlog \
        -text "Show Log" \
         -highlightbackground [c_dblue] \
        -bg [c_aqua] -bd 1 -font $BFONT \
        -command {reader.cb_b_showlog}
  button $W.b_quit    \
        -text "Quit"            \
         -highlightbackground [c_dblue] \
        -bg [c_aqua] -bd 1 -font $BFONT \
        -command {reader.cb_b_quit}
  pack  $W.b_view              \
        $W.b_copy              \
	$W.b_help              \
        $W.b_showlog           \
        $W.b_quit              \
        -side left -expand 1
}

#................................................................#
proc reader.cb_b_view {} {
  global READG
  global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT
   
  reader.save_user_defaults $READG(DEFAULTS_PATH)
  set STATUS [check_if_selected]
  if { $STATUS != 0 } {
     return 
  }
  set FPATH [get_li_file_path $READG(CUR_INDEX)]
  set EXTENSION [file extension $FPATH]
  tclu_log_message "Viewing document $FPATH"     
  switch -- $EXTENSION {
     ".pdf" {
       if { $READG(OS) != "nt" } {
          set COMMAND_LIST [split $READG(ACROBAT_COMMAND)]
          set COMMAND [lindex $COMMAND_LIST 0]
       } else {
          set COMMAND_LIST $READG(ACROBAT_COMMAND)
          set COMMAND $READG(ACROBAT_COMMAND)
       }
       if { ! [tclu_is_path_executable $COMMAND] } {
          set STATUS [install_acrobat_reader COMMAND]
	  if { $STATUS != 0 } {
	    return 
	  }
	  set NEW_COMMAND_LIST $COMMAND
	  if { [llength $COMMAND_LIST] > 1 } {
	     set NEW_COMMAND_LIST \
	        "$NEW_COMMAND_LIST [lrange $COMMAND_LIST 1 end]"
	  }
	  set READG(ACROBAT_COMMAND) $NEW_COMMAND_LIST
          set CHOICE [tku_popup_message info okcancel \
	       "Press OK when installation is complete !"]
          if { $CHOICE != "ok" } {
             return 
          }   
       }
       if { $READG(OS) == "nt" } {
          set ACROBAT_COMMAND [tclu_path_back_to_fore $READG(ACROBAT_COMMAND)]
          regsub -all / $FPATH "\\\\\\\\" FPATH
          set COMMAND "\"$ACROBAT_COMMAND\" $FPATH"
       } else {
          set COMMAND "$READG(ACROBAT_COMMAND) $FPATH &"
       }
       puts "$COMMAND"
	 if { [catch {eval "exec $COMMAND"} RESPONSE]} {
            if {$READG(OS) != "nt" } {
               tku_popup_message error ok $RESPONSE
            }
       } else {
         tku_message "Launching Acrobat Reader ; Please wait ..."
         tku_cursor_watch       
         after 5000
         tku_message " "
         tku_cursor_normal      
       }
     }
     ".txt" {
        set TMP_PATH [tclu_get_tmp_name "reader"]
        copy_file_with_tags $FPATH $TMP_PATH
        tku_popup_file $TMP_PATH
        catch {file delete $TMP_PATH}
     }
     default {
         tku_popup_message error ok \
	     "No viewing program matches the extension\
	    \nof this file ($EXTENSION)"
     }	
  }
}

#................................................................#
proc install_acrobat_reader { ACRO_COMMAND_P } {
   global READG
   upvar $ACRO_COMMAND_P ACRO_COMMAND
      
   set CHOICE [tku_popup_message question okcancel \
      "The specified path: $ACRO_COMMAND\
     \ndoes not point to a valid executable program.\
     \n\
     \nIf you have already installed Acrobat Reader in a different path, \
     \nclick 'Cancel' and change the path (located in the\
     \n'Acrobat Reader Command' field) to the correct one.\
     \n\
     \nIf you never installed Acrobat Reader, you may continue to \
     \ninstall Acrobat Reader now.\
     \n\
     \nDo you wish to install Acrobat Reader now ?"]
   if { $CHOICE != "ok" } {
      return 1
   }
   
   set OS $READG(OS)
   if { $OS == "nt" } {
      set STATUS [install_acrobat_reader_nt ACRO_COMMAND]
   } else {
      set STATUS [install_acrobat_reader_unix ACRO_COMMAND]
   }
   return $STATUS
}
#................................................................#
proc install_acrobat_reader_nt { ACRO_COMMAND_P } {
   global READG
   upvar $ACRO_COMMAND_P ACRO_COMMAND
      
   set OS $READG(OS)
   set EXE_PATH [file join [pwd] helpers $OS acrdsrch.exe]
   if { ! [file exists $EXE_PATH] } {
      tku_popup_message error ok \
	 "Sorry, The file $EXE_PATH\
	\ndoes not exist"
      return 1
   }
   set DEST_DIR " c:/Progra~1/Adobe/Acroba~1.0/Reader"
   set CHOICE [tku_popup_message info okcancel \
      "Please specify below the desired path for the Acrobat Reader.\
     \nSubsequently, you will be transferred to the Adobe Acrobat Reader\
     \nstandard installation process. You will have to accept the license\
     \nagreement and re-specify the directory into which the reader is installed.\
     \nPlease make sure it is the same as you specified below!" \
	       "" DEST_DIR]
   if { $CHOICE != "ok" } {
      return 1 
   }
   if [catch {eval "exec $EXE_PATH &"} RESPONSE] {
      tku_popup_message error ok $RESPONSE
      return 1 
   }
   set ACRO_COMMAND [file join $DEST_DIR acrord32]
   
   return 0
}

#................................................................#
proc install_acrobat_reader_unix { ACRO_COMMAND_P } {
   global READG
   upvar $ACRO_COMMAND_P ACRO_COMMAND
         
   set IS_ROOT [tku_is_root warning]
   if { ! $IS_ROOT } {
      return 1
   }
      
   set OS $READG(OS)
   if { $OS == "hp10" } {
      set OS "hp9"
   }
   set DATA_PATH [file join [pwd] helpers $OS acrdsrch.tgz]
   if { ! [file exists $DATA_PATH] } {
      tku_popup_message error ok \
	 "Sorry, The file $DATA_PATH\
	\ndoes not exist"
      return 1
   }
      
   set DEST_TMP_PATH [tclu_get_tmp_name acrobat]
   set STATUS [tclu_mkdir $DEST_TMP_PATH 1]
   if {$STATUS != 0} { 
      return 1 
   }
   
   set CUR_DIR [pwd]
   if [ catch {cd $DEST_TMP_PATH} RESPONSE ] {
       tku_popup_message error ok $RESPONSE
       return 1
   }
      
   set PROGRESS_PATH [tclu_get_tmp_name tmp_reader]
   set COMMAND [format "%s %s %s | %s | %s %s" \
          [file join $READG(TOOLS_PATH) pipe_in.exe] \
	  "$DATA_PATH" \
          $PROGRESS_PATH    \
          [file join $READG(TOOLS_PATH) gunzip.exe] \
	  $READG(TAR) "xf -"]
   set TITLE "Uncompressing Acrobat Reader..."
   set STATUS [tku_long_command $COMMAND $TITLE $READG(W_MESSAGE) \
              abort $PROGRESS_PATH]
   catch { file delete $PROGRESS_PATH } 
   if { $STATUS != "ok" } {
      catch {cd $CUR_DIR}
      catch {file delete -force $DEST_TMP_PATH}
      return 1
   }
   set DEST_DIR "/usr/Acrobat3"
   set CHOICE [tku_popup_message info okcancel \
      "Please specify below the desired path for the Acrobat Reader.\
     \nSubsequently, you will be transferred to the Adobe Acrobat Reader\
     \nstandard installation which runs in a separate terminal window.\
     \nYou will have to accept the license agreement and re-specify\
     \nthe directory into which the reader is installed.\
     \nPlease make sure it is the same as you specified below!" \
	       "" DEST_DIR]
   if { $CHOICE != "ok" } {
      catch {cd $CUR_DIR}
      catch {file delete -force $DEST_TMP_PATH}
      return 1 
   }
   set SCRIPT_PATH [file join $DEST_TMP_PATH INSTALL]
   set SH_COMMAND "$READG(XTERM) $SCRIPT_PATH"
   puts $SH_COMMAND
   if [catch {eval "exec $SH_COMMAND"} RESPONSE] {
      tku_popup_message error ok $RESPONSE
      catch {cd $CUR_DIR}
      catch {file delete -force $DEST_TMP_PATH}
      return 1 
   }
   set ACRO_COMMAND [file join $DEST_DIR bin acroread]
   catch {cd $CUR_DIR}
   catch {file delete -force $DEST_TMP_PATH}
   
   return 0
}
#................................................................#
proc reader.cb_b_copy {} {
  global READG
  global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT

  set STATUS [check_if_selected]
  if { $STATUS != 0 } {
     return 
  }
  set FPATH [get_li_file_path $READG(CUR_INDEX)]
  set TAIL [file tail $FPATH]
       
  tku_cursor_watch       
  tku_restore_tk_getOpenFile 
  set DEST_PATH \
     [tk_getSaveFile -initialdir  $READG(COPY_DIR) \
                     -initialfile $TAIL]
  tku_override_tk_getOpenFile 
  tku_cursor_normal      
  if { [string length $DEST_PATH] > 0 } { 
       set READG(COPY_DIR) [file dirname $DEST_PATH]
       if [ file exists $DEST_PATH ] {
	  if [ tclu_is_path_dir $DEST_PATH ] {
	     set DEST_PATH [file join $DEST_PATH $TAIL]
	  }
       }
       set MESSAGE "Copy file $TAIL to $DEST_PATH ? "
       set CHOICE [tku_popup_message question okcancel $MESSAGE]
       if { $CHOICE == "ok" } {
          tku_message "Copying file ; Please wait ..."
          tclu_log_message "copy $FPATH to $DEST_PATH"     
	  puts "copy $FPATH to $DEST_PATH"
          tku_cursor_watch  
          set EXTENSION [file extension $FPATH]
          switch -- $EXTENSION {  
             ".txt" { 
	          copy_file_with_tags $FPATH $DEST_PATH       
               }
             default {
	         if { [catch { file copy -force $FPATH $DEST_PATH } RESPONSE ] } {
	           tku_popup_message error ok $RESPONSE
	         }
	       }
	  }
          tku_cursor_normal      
	  tku_message " "	  
       }	  
  }
}

#................................................................#
proc copy_file_with_tags { IPATH OPATH } {
  global READG
  
   set HOSTID [tclu_get_hostid $READG(OS)]
   set DATE [clock format [clock seconds] -format "%d %b %Y"]   
   
   if {[catch {open $IPATH r} IFP]} {
      tku_popup_message error ok $IFP
      return 1
   }
   if {[catch {open $OPATH w} OFP]} {
      close $IFP
      tku_popup_message error ok $OFP
      return 1
   }
   while {! [eof $IFP]} {
     gets $IFP LINE
     regsub -all "@N@" $LINE $HOSTID LINE1
     regsub -all "@D@" $LINE1 $DATE  NEWLINE
     puts $OFP $NEWLINE
   }
   close $IFP
   close $OFP
   
   return 0
}


#................................................................#
proc check_if_selected {} {
  global READG
     
  if { $READG(CUR_INDEX) < 0 || $READG(CUR_INDEX) >= $READG(NUM_TITLES) } {
     tku_popup_message error ok "No entry is selected"
     return 1
  }
  return 0
}
#................................................................#
proc reader.cb_b_help {} {
  global READG 
   
  set PATH [file join $READG(INSTALL_PATH) reader.hlp]
  
  tku_popup_file $PATH start "-height 25"
}

#................................................................#
proc reader.cb_b_showlog {} {
  global READG 
   
   tku_popup_file $READG(LOG_PATH) end
}

#................................................................#
proc reader.cb_b_quit {} {
  global READG 
  set CHOICE yes
  set CHOICE [tku_popup_message warning yesno "Really Quit ?"]
  if { $CHOICE == "yes" } { 
    tclu_log_message "End Session."  "underline" 
     
    reader.save_user_defaults $READG(DEFAULTS_PATH)
    destroy .  
    exit
  }
}
##################################################################
proc  reader.update_display {} {
   global READG
   global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT

   $READG(W_TITLES) delete 0 end
   set CUR_TYPE    [string tolower $READG(TYPE)]
   
   set READG(TITLES_LIST) [list]
   for {set INDEX 0} { $INDEX < $READG(NUM_TITLES) } { incr INDEX } {
      set TYPE     $LI_TYPE($INDEX)
      set PRODUCTS $LI_PRODUCTS($INDEX)
      if { $TYPE != $CUR_TYPE } {
          continue 
      }
      set ADD_IT 0
      foreach PRODUCT $PRODUCTS {
	 set UC_PRODUCT [string toupper $PRODUCT]
         if { ! [info exists READG($UC_PRODUCT)] } {
	   continue 
	 }
	 if { $READG($UC_PRODUCT) != 0 } {
	   set ADD_IT 1
	   break 
	 }
      }
      if { $ADD_IT } {
         lappend READG(TITLES_LIST) "$LI_NAME($INDEX) ##$INDEX"
      }
   }
   set READG(TITLES_LIST) [lsort $READG(TITLES_LIST)]
   set COUNT 0
   foreach NAME $READG(TITLES_LIST) {
      set TAIL_INDEX [string last "##" $NAME]
      $READG(W_TITLES) insert end \
         [string range  $NAME 0 [expr $TAIL_INDEX-1]]
      incr COUNT
   }
   switch -- $COUNT {
     0 {
       set MESSAGE "No titles available"
     }
     1 {
       set MESSAGE "one title is available"
     }
     default {
       set MESSAGE "$COUNT titles are available"
     }
   }
   $READG(W_LCOUNT) configure -text $MESSAGE
}

##################################################################
proc  reader.update_abstract {} {
   global READG
   global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT
   
   if { [string length $READG(W_ABSTRACT)] == 0 } {
      return 
   }
   $READG(W_ABSTRACT) delete 1.0 end      
   $READG(W_ABSTRACT) insert 1.0 "\n"      
   set SEL_LIST [$READG(W_TITLES) curselection]
   if { [llength $SEL_LIST] != 1 } {
      return 
   }
   set SEL_ITEM [lindex $SEL_LIST 0]
   set NAME [lindex $READG(TITLES_LIST) $SEL_ITEM]
   set TAIL_INDEX [string last "##" $NAME ]
   set INDEX [string range $NAME [expr $TAIL_INDEX+2] end]
   set READG(CUR_INDEX) $INDEX
   set ABSTRACT $LI_ABSTRACT($INDEX)
   set FIRST_NL [string first "\n" $ABSTRACT]
   $READG(W_ABSTRACT) insert 1.0 \
      "[string range $ABSTRACT 0 [expr $FIRST_NL-1] ]\n" title
   $READG(W_ABSTRACT) insert end \
      [string range $ABSTRACT [expr $FIRST_NL+1] end]
}      

##################################################################
proc get_li_file_path { INDEX } {
   global READG
   global LI_NAME LI_TYPE LI_FILE LI_PRODUCTS LI_ABSTRACT
      
   set TYPE [string tolower $LI_TYPE($INDEX)]
   regsub -all { } $TYPE {_} DIR
   set FILE $LI_FILE($INDEX)
   set PATH [file join $READG(LIT_PATH) $DIR $FILE]
   return $PATH
}

##################################################################
#
# This procedure fills the paths to useful UNIX commands in the
# global variable READG according to the operating system on which
# the script is running on.
#
# The commands are:
#  cat
#  chmod
#  cp
#  dd
#  tar
#
proc reader.set_unix_commands {} {
global READG
   set COMMANDS [list CAT CHMOD CP DD TAR WHOAMI]
   set XTERM_OPTIONS "-sb -sl 400 -e /bin/sh"
   
   switch -- $READG(OS) {
     aix {
        set READG(CAT)   "/bin/cat"
        set READG(CHMOD) "/bin/chmod"
        set READG(CP)    "/bin/cp"
        set READG(DD)    "/bin/dd"
        set READG(TAR)   "/bin/tar"
        set READG(WHOAMI) "/bin/whoami"
	set READG(XTERM)  "/usr/bin/X11/xterm $XTERM_OPTIONS"
     } 
     hp10 {
        set READG(CAT)   "/bin/cat"
        set READG(CHMOD) "/bin/chmod"
        set READG(CP)    "/bin/cp"
        set READG(DD)    "/bin/dd"
        set READG(TAR)   "/bin/tar"
        set READG(WHOAMI) "/bin/whoami"
	set READG(XTERM)  "/usr/bin/X11/xterm $XTERM_OPTIONS"
     }
     nt {
        set READG(CAT)   [file join $READG(TOOLS_PATH) cat.exe]
        set READG(CHMOD) [file join $READG(TOOLS_PATH) chmod.exe]
        set READG(CP)    [file join $READG(TOOLS_PATH) cp.exe]
        set READG(DD)    [file join $READG(TOOLS_PATH) dd.exe]
        set READG(TAR)   [file join $READG(TOOLS_PATH) tar.exe]
	# No need on NT
        set READG(WHOAMI) ""  
	set READG(XTERM)  ""
     }
     solaris {
        set READG(CAT)   "/bin/cat"
        set READG(CHMOD) "/bin/chmod"
        set READG(CP)    "/bin/cp"
        set READG(DD)    "/bin/dd"
        set READG(TAR)   "/bin/tar"
        set READG(WHOAMI)   "/usr/ucb/whoami"
	set READG(XTERM)  "/usr/openwin/bin/xterm $XTERM_OPTIONS"
     }
     linux {
        set READG(CAT)   "/bin/cat"
        set READG(CHMOD) "/bin/chmod"
        set READG(CP)    "/bin/cp"
        set READG(DD)    "/bin/dd"
        set READG(TAR)   "/bin/tar"
        set READG(WHOAMI) "/usr/bin/whoami"
	set READG(XTERM)  "/usr/bin/X11/xterm $XTERM_OPTIONS"
     }
     default {
        puts "Illegal operating system: $READG(OS)"
	exit 1	
     }
   }
   foreach COMMAND $COMMANDS {
        if { $READG($COMMAND) == "" } {
	   continue 
	}
        if { ! [file exists $READG($COMMAND)] } {
           puts "The command $READG($COMMAND) does not exist."
	   puts "Please remedy and rerun."
	   exit 1 
	   }
        if { ! [file executable $READG($COMMAND)] } {
           puts "The command $READG($COMMAND) is not an executable file."
	   puts "Please remedy and rerun."
	   exit 1 
	}
   }     
   tclu_set_commands \
     $READG(CAT) $READG(CHMOD) $READG(CP) $READG(DD) $READG(TAR) $READG(WHOAMI)    
}

################ PATH CREATION ROUTINES #######################
set  READG(TOOLS_PATH)    [lindex $argv 0]
set  READG(INSTALL_PATH)  [file join [pwd] install]
set  READG(OS)            [file tail $READG(TOOLS_PATH)] 
if { $READG(OS) == "nt" } {
  set READG(ACROBAT_COMMAND) "c:/Progra~1/Adobe/Acroba~1.0/Reader/acrord32.exe"
  console show
}
puts " Literature Reader Version $READG(VERSION)"
source [file join $READG(INSTALL_PATH) tclu.tcl]
source [file join $READG(INSTALL_PATH) tku.tcl]
reader.main


