#---------------------------------------------------
#
#  Module : tku.tcl
#  Author : MK
#  Date   : 20 Aug 1997
#
#  Description :
#
#  This module provides Tk utilities needed by the Frontline Pro-Installer
#  module and the various installation plug-ins.
#  All the public procedures are called tku_<name>
#  All the private procedures are called tku.<name> - should never be called
#
#  List of public procedures in this module:
#
#  tku_set_tools_dir  -|---- Initialization routines
#  tku_set_os         -|
#  tku_long_command   - Perform long commands allowing user stop
#  tku_message        - Writes a message to the message line
#  tku_override_tk_getOpenFile - Changes slightly the builtin tk_getOpenFile
#  tku_popup_file     -	Opens a popup displaying a file in it
#  tku_popup_log_open -	| 
#  tku_popup_log      -	| Routines for opening a logging popup and adding data	
#  tku_popup_log_show - | to it
#  tku_is_root        - Pops up a 'you are not root' message
#  tku_popup_message  -	Pops up a dialog window in various modes
#  tku_cursor_watch   - Changes the cursor to a timepiece and updates
#  tku_cursor_normal  - Changes the cursor to left pointer and updates
#  tku_center         - Finds the center of the main window
#  tku_center_on_window - Position one window on top of another
#  tku_scrolled_listbox - Creates a list box w/2 scrollbars
#  tku_color - 		Converts rrggbb to Tk color scheme
#  c_<name> -		Common colors creation
#  tku_font - 		Converts xyznn fonts to Tk font scheme
#  xyznn - 		Common font creation
#---------------------------------------------------
#
#  Change History 
#  --------------
#  970820 - First Version
#  970906 - Documentation
#
set    TKU_G(WCOUNTER)     0
set    TKU_G(AFTER_ID)     -1
set    TKU_G(BLINK_STATE)  -1
set    TKU_G(BLINK_OFFSET) 0 
set    TKU_G(COMMAND_MESSAGE)  "" 
set    TKU_G(COMMAND_MODE)     "abort" 
set    TKU_G(COMMAND_LOG_PATH) "" 
set    TKU_G(COMMAND_LOG_FP)   "" 
set    TKU_G(COMMAND_PERCENT) 0 
set    TKU_G(COMMAND_DONE) 0
set    TKU_G(GRAB_CURRENT) ""
set    TKU_G(MESSAGE_RES)  ""
set    TKU_G(PROG_FILE)    ""
set    TKU_G(STOP_FILE)    ""
set    TKU_G(LOCAL_STOP_FILE)    ""
set    TKU_G(INPUT_PIPE)   ""
set    TKU_G(COMMAND_WMESSAGE) ""
set    TKU_G(ENTRY_VAR)    ""
set    TKU_G(COPY_DIR)     ""

#*******************************************************
#
# Name :	tku_set_tools_dir
#
# Description :	
#   Sets the locations of the tools directory 
#
# Input	:	TOOLS_DIR - location of the tools
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tku_set_tools_dir {TOOLS_DIR} {
  global TKU_G
  
  set TKU_G(TOOLS) $TOOLS_DIR
}

#*******************************************************
#
# Name :	tku_set_os
#
# Description :	
#   Sets the operating system
#
# Input	:	OS - name of operating system
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tku_set_os {OS} {
  global TKU_G
  
  set TKU_G(OS) $OS
}

################ LONG COMMANDS EXECUTION #########################
#*******************************************************
#
# Name :	tku_long_command
#
# Description :	
#   Executes a long command, showing the caller a progress message
#   and letting him stop the process.
#
# Input	:	COMMAND - The command to process
#               MESSAGE - The message to display while the command executes
#               WMESSAGE - The widget in which to display the message and the
#                         stop button
#               MODE    - abort  - Abort if command produced any output
#                         choice - Show output and give user choice
#                         ignore - Do not show output and continue
#               PROG_FILE - Name of a progress file to display in message line
#               STOP_FILE - Name of a file to stop the comamnd on
# Output :	None
# Return Value:	ok for success
#               error for command error
#               cancel for operator stop
#*******************************************************
#
proc tku_long_command { COMMAND MESSAGE {WMESSAGE ""}
			{MODE "abort"} {PROG_FILE ""} {STOP_FILE ""} } {
  global TKU_G
  set LOG_TEMP_PATH  [file join [tclu_get_home] .valor_install log_temp]  

  set TKU_G(COMMAND_MODE) $MODE 
  set TKU_G(COMMAND_MESSAGE) $MESSAGE 
  set TKU_G(PROG_FILE) $PROG_FILE
  set TKU_G(STOP_FILE) $STOP_FILE
  
  #
  # NT backslashes
  #
  set COMMAND1 [tclu_path_back_to_fore $COMMAND]
  set COMMAND $COMMAND1
  
  if { [string length $WMESSAGE] == 0 } {
    set WMESSAGE $TKU_G(COMMAND_WMESSAGE)
  }
  #
  # Progress file handling
  #
  if { [string length  $TKU_G(PROG_FILE)] > 0} {
    catch { file delete $TKU_G(PROG_FILE) } 
  }
  
  #
  # External stop file file handling
  #
  if { [string length  $TKU_G(STOP_FILE)] > 0} {
    if [tclu_file_exists $TKU_G(STOP_FILE)] {
      file delete $TKU_G(STOP_FILE)
    }
  }
  #
  #  Preparing the local command log
  #
  set LOG_PATH [tclu_log_get_path]
  set TKU_G(COMMAND_LOG_PATH) ${LOG_PATH}.tmp  
  if [tclu_file_exists $TKU_G(COMMAND_LOG_PATH)] {
    file delete $TKU_G(COMMAND_LOG_PATH)
  }
  if [catch {open $TKU_G(COMMAND_LOG_PATH) {RDWR CREAT} 0777} \
	  TKU_G(COMMAND_LOG_FP)] { 
    puts "$TKU_G(COMMAND_LOG_FP)"
    set TKU_G(COMMAND_LOG_FP) -1 
  }   
  #
  #  Local stop file is used by the pipe_out executable to signal
  #  that the background ended. This is due to some NT TCL/TK 
  #  limitations.
  #
  set TKU_G(LOCAL_STOP_FILE) [tclu_get_tmp_name tku_local_stop]
  if [tclu_file_exists $TKU_G(LOCAL_STOP_FILE)] {
      file delete -force $TKU_G(LOCAL_STOP_FILE)
  }
  #
  #  Prepare the pipe_out executable suffix
  #
  set PIPE_OUT_EXE [file join $TKU_G(TOOLS) pipe_out.exe]
  set PIPE_OUT_COMMAND \
       "$PIPE_OUT_EXE $TKU_G(COMMAND_LOG_PATH) $TKU_G(LOCAL_STOP_FILE)"
  #
  #  ################ Command is executed here !!!!!!!!!!!!!!
  #
  tclu_log_message [format "Long Command:\n%s" [tclu_break_line $COMMAND 60]]
  set TKU_G(COMMAND_DONE) ""
  if [catch {open "|$COMMAND |& $PIPE_OUT_COMMAND"} RESPONSE] {
    set WLOGS \
	[tku_popup_log_open "Command Failure" \
	     "-height 12"]
    set WLOG [lindex $WLOGS 0]
    set WT   [lindex $WLOGS 1]
    tku_popup_log $WT normal "[tclu_break_line $COMMAND 60]\n"
    tku_popup_log $WT error  " [tclu_break_line " --> $RESPONSE" 60]\n"
    tclu_log_message [tclu_break_line "--> $RESPONSE" 60] "error"
    return "error"
  } else {
    . configure -cursor watch
    set TKU_G(INPUT_PIPE) $RESPONSE
    fconfigure $TKU_G(INPUT_PIPE) -blocking 0
    set W_STOP_FRAME \
	[tku.stop_popup $WMESSAGE]
    while { $TKU_G(COMMAND_DONE) == "" } {
       vwait TKU_G(COMMAND_DONE)
    }
  }
  if { $TKU_G(COMMAND_DONE) == "error"} {
    switch -- $TKU_G(COMMAND_MODE) {
       "abort" {
         set MESSAGE "The command :\
 	         \n\n[tclu_break_line $COMMAND 60]\
	         \n\nproduced error messages."
	 tku_popup_message error ok $MESSAGE $TKU_G(COMMAND_LOG_PATH)	 
       }
       "choice" {
         set MESSAGE "The command :\
 	         \n\n[tclu_break_line $COMMAND 60]\
	         \n\nproduced error messages.\
	       \n\nAre you sure you wish to continue?"	       
         set CHOICE [tku_popup_message warning yesno $MESSAGE
		    $TKU_G(COMMAND_LOG_PATH) ]
         switch -- $CHOICE {
           "yes" {
   	     set TKU_G(COMMAND_DONE) ok
	    }
	   "no" {
	     set TKU_G(COMMAND_DONE) error
	    }
            default { bgerror "Switch error tku.tcl (0)" }
         }
       }
       default {
       }
     }
   }
   return $TKU_G(COMMAND_DONE)
}

proc tku_long_command_set_wmessage {WMESSAGE} {
  global TKU_G
  
  set TKU_G(COMMAND_WMESSAGE) $WMESSAGE
}

proc tku_message { TEXT } {
  global TKU_G
  if { $TKU_G(COMMAND_WMESSAGE) != "" } {
     $TKU_G(COMMAND_WMESSAGE) configure -text $TEXT
     update idletasks
  } else {
     puts $TEXT 
  }
}
  
#................................................................#
proc tku.stop_popup {WMESSAGE} {
  global TKU_G
  
  if { [string length $WMESSAGE] == 0 } {
    return ""
  }
  
  set W_STOP_FRAME $WMESSAGE.frame
  
  if { ! [winfo exists $W_STOP_FRAME] } {
    frame $W_STOP_FRAME -bg [c_yellow] 
    label $W_STOP_FRAME.l -bg [c_yellow] -font [tbr14] -justify left -bd 1
    label $W_STOP_FRAME.p -bg [c_yellow] -font [tbr14] -justify left -bd 1 \
	-text "  0%"
    button $W_STOP_FRAME.b -text "Stop" -bg [c_aqua] -font [tbr14] \
	-highlightbackground [c_yellow] \
	-command [list tku.stop_long_command \
		      "cancel" $W_STOP_FRAME ]
    pack $W_STOP_FRAME.b -side right 
    pack $W_STOP_FRAME.p -side right 
    pack $W_STOP_FRAME.l -side left -fill x
  }
  $W_STOP_FRAME.b configure -text "Stop"
  $W_STOP_FRAME.p configure -text " "
  $W_STOP_FRAME.l configure -text $TKU_G(COMMAND_MESSAGE)
  pack $W_STOP_FRAME  -expand 1 -fill x
  catch {grab $W_STOP_FRAME}
  
  set TKU_G(BLINK_STATE)  0
  set TKU_G(BLINK_OFFSET) 0
  tku.long_command_blink $W_STOP_FRAME [string length $TKU_G(COMMAND_MESSAGE)]
  return $W_STOP_FRAME
}

#................................................................#
proc tku.long_command_blink {W LEN } {
  global TKU_G

  set RENEW_TIMER 1
  
  switch -- $TKU_G(BLINK_STATE) {
    {0} {
      set COLOR [c_red]
    }
    {1} {
      set COLOR [c_aqua]
    }
    default { bgerror "Switch error tku.tcl (1)" }
  }
  $W.b configure -bg $COLOR
  set PROGRESS "    "
  if [tclu_file_exists $TKU_G(PROG_FILE)] {
    if { ![catch {open $TKU_G(PROG_FILE) RDONLY} FP] } {
      set PROGRESS [read $FP]
      catch {close $FP}
    }
  }
  $W.p configure -text $PROGRESS
  update idletasks
  
  set TKU_G(BLINK_STATE) [expr 1-$TKU_G(BLINK_STATE)]
  incr TKU_G(BLINK_OFFSET)
  if { $TKU_G(BLINK_OFFSET) > 60 } {
    set TKU_G(BLINK_OFFSET) 0 
  }

  #
  # STOP_FILEs are used by commands which read from DAT to make a long
  # tar stop
  if { [tclu_file_exists $TKU_G(STOP_FILE)] } {
    file delete -force $TKU_G(STOP_FILE)
    tku.stop_long_command "end" $W      
    set RENEW_TIMER 0
  } 
  #
  # LOCAL_STOP_FILEs are used by any command. They are written by the
  # pipe_out program (which is suffixed to all long commands) when the
  # input pipe to pipe_out is closed.
  #
  if { [tclu_file_exists $TKU_G(LOCAL_STOP_FILE)] } {
    # stop file was found, we simulate a normal completion
    catch {file delete -force $TKU_G(LOCAL_STOP_FILE)} 
    tku.stop_long_command "end" $W      
    set RENEW_TIMER 0
  } 
  
  if { $RENEW_TIMER } {
    set TKU_G(AFTER_ID) \
	[after 500 [list tku.long_command_blink $W $LEN]]   
  }
}

#................................................................#
proc tku.stop_long_command \
   {END_CONDITION W_STOP_FRAME} {
   global TKU_G
      
   set COMMAND_DONE "ok"
   if { $TKU_G(AFTER_ID) != -1 } {
     after cancel $TKU_G(AFTER_ID)
   }
   if {$END_CONDITION == "cancel" && [string length $W_STOP_FRAME] > 0 } {
      $W_STOP_FRAME.b configure -text "Stopping..."
      update idletasks
   }
   catch {close $TKU_G(COMMAND_LOG_FP)}
   if [tclu_file_exists $TKU_G(COMMAND_LOG_PATH)] {
      if { [file size $TKU_G(COMMAND_LOG_PATH)] > 0 } {
	 set COMMAND_DONE "error"
	 set LOG_MODE     "error"
         if { $TKU_G(COMMAND_MODE) == "ignore" } { 
	    set COMMAND_DONE "ok"
	    set LOG_MODE     "normal"
	 }
         tclu_log_message "Command produced messages below:" $LOG_MODE
         catch {tclu_log_file $TKU_G(COMMAND_LOG_PATH)}
         tclu_log_message "End of command message" $LOG_MODE	 
      }
   }
   set PIDS [split [pid $TKU_G(INPUT_PIPE)]]
   foreach PID $PIDS {
     if [catch {exec kill $PID} RESP] {
#        puts $RESP
     }
   }
   catch {close $TKU_G(INPUT_PIPE)}      
   catch {pack forget $W_STOP_FRAME}
   catch {grab release $W_STOP_FRAME}
   if { [string length  $TKU_G(PROG_FILE)] > 0} {
     if [tclu_file_exists $TKU_G(PROG_FILE)] {
        file delete -force $TKU_G(PROG_FILE)
     }
   }
   if { [tclu_file_exists $TKU_G(LOCAL_STOP_FILE)] } {
      catch {file delete -force $TKU_G(LOCAL_STOP_FILE)}
   }
   
   if {$END_CONDITION == "cancel"} {
      tclu_log_message "Command canceled by user" "error"
      set COMMAND_DONE cancel
   } else {
      if { $COMMAND_DONE == "ok" } {
         tclu_log_message "Command ended normally" ""
         set COMMAND_DONE ok
      }
   }
   . configure -cursor left_ptr
   set TKU_G(COMMAND_DONE) $COMMAND_DONE
}

################ Override tk_getOpenFile #########################
#*******************************************************
#
# Name :	tku_override_tk_getOpenFile
#
# Description :	
#   The Tk standard tk_getOpenFile does not allow to select a directory;
#   only a file. This command changes this behaviour by making the 'Open'
#   button closing the window and returning the current selected directory.
#
# Input	:	None
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tku_override_tk_getOpenFile {} {
 global TKU_G
 global tk_library
 source [file join $tk_library tkfbox.tcl]
 rename tkFDialog_OkCmd tkFDialog_OkCmd.org

 proc tkFDialog_OkCmd.new w {
#   upvar #0 $w data

    # set text [tkIconList_Get $data(icons)]
    set data_icons [lindex [uplevel #0 array get [winfo name $w] icons] 1]
    set text [tkIconList_Get $data_icons]

    # tkFDialog_Done $w [tkFDialog_JoinFile $data(selectPath) $text]
    set data_selectpath [lindex [uplevel #0 array get [winfo name $w] selectPath] 1]
    tkFDialog_Done $w [tkFDialog_JoinFile  $data_selectpath $text]
 }
 proc tk_getOpenFile.new {{ARG1 ""} {ARG2 ""} \
                          {ARG3 ""} {ARG4 ""} \
                          {ARG5 ""} {ARG6 ""} \
                          {ARG7 ""} {ARG8 ""} \
                          {ARG9 ""} {ARG10 ""} \
                          {ARG11 ""} {ARG12 ""} } {
     set COMMAND  "tkFDialog $ARG1 $ARG2 $ARG3 $ARG4  $ARG5  $ARG6 \
                   $ARG7 $ARG8 $ARG9 $ARG10 $ARG11 $ARG12"
     return [eval $COMMAND]
 }
 
 rename tkFDialog_OkCmd.new tkFDialog_OkCmd
 if { $TKU_G(OS) == "nt" } {
     if [catch {rename tk_getOpenFile tk_getOpenFile.org} ERROR] {
        puts "1. $ERROR"
     }
     if [catch {rename tk_getOpenFile.new tk_getOpenFile} ERROR] {
        puts "2. $ERROR"
     }
 }
 return
}

proc tku_restore_tk_getOpenFile {} {
 global TKU_G
 global tk_library
 rename tkFDialog_OkCmd tkFDialog_OkCmd.new
 rename tkFDialog_OkCmd.org tkFDialog_OkCmd
 
 return
}


################ POPUP FILE #########################
#*******************************************************
#
# Name :	tku_popup_file
#
# Description :	
#   Opens a window showing a file in it. If the file is longer than
#   SIZE_LIMIT, only SIZE_LIMIT bytes are displayed (rounded to a full
#   line). Tags are supported inside the file:
#    normal (default) - font: cbr14  color: black
#    error            - font: cbr14  color: red
#    underline        - font: cbr14  color: black  underline
#    title            - font: cbr18  color: black  underline
#   tags should appear as ()tag in the beginning of the line.
#
# Input	:	PATH - Path of file to display
#               POSITION - start for showing line 1 on top
#                          end for showing last line on bottom
#               ATTRIBUTES - Allows additional attribute (e.g. -height 40)
# Output :	None
# Return Value:	None
#
#*******************************************************
#
proc tku_popup_file {PATH {POSITION start} {ATTRIBUTES ""}} {
   global TKU_G
   set WF .popup_file
   append WF _ $TKU_G(WCOUNTER)
   set WFT $WF.text
   set WFB $WF.button
   set SIZE_LIMIT 100000

   if { ! [tclu_is_path_file $PATH] } {
      tku_popup_message warning ok [format "Missing file:\n%s" $PATH]
      return
   }
   set WLOGS [tku_popup_log_open $PATH $ATTRIBUTES]
   set W  [lindex $WLOGS 0]
   set WT [lindex $WLOGS 1]
          
   set NORMAL_STRING ""
   set FP [open $PATH]
   set SIZE [file size $PATH]
   if {$SIZE > $SIZE_LIMIT} {
      seek $FP -$SIZE_LIMIT end
      gets $FP LINE
      set LINE_SIZE [string length $LINE]
      tku_popup_log $WT error \
	 [format "File %s is too long.\
	        \n%d initial bites were not displayed\n" \
	         $PATH [expr $SIZE - $SIZE_LIMIT + $LINE_SIZE]]
   }      
   set LINE_NUM 0
   while {![eof $FP]} {
      gets $FP LINE
      incr LINE_NUM
      if {[string range $LINE 0 1] == "()"} {
         tku_popup_log $WT normal "$NORMAL_STRING"
	 set NORMAL_STRING ""
	 set I1 [string wordend $LINE 2]
	 tku_popup_log $WT [string range $LINE 2 [expr $I1-1]] \
	                   "[string range $LINE [expr $I1+1] end]\n"
      } else {   
         append NORMAL_STRING $LINE "\n"
      }
   }
   tku_popup_log $WT normal "$NORMAL_STRING"
   close $FP
      
   if {$POSITION == "end"} {
      $WT yview moveto 1
   }
   tku_popup_log_show $W
}

################ POPUP LOG #########################
#*******************************************************
#
# Name :	tku_popup_log_open
#
# Description :	
#   Opens a window to which lines can be added in sequence using the 
#   procedure tku_popup_log. The window is only displayed upon call to 
#   tku_popup_log_show. 
#
# Input	:	NAME - Title of window
#               ATTRIBUTES - Allows additional attribute (e.g. -height 40)
# Output :	None
# Return Value:		A list of 2 widgets - 
#                W     - Widget of window
#                WTEXT - Widget of text portion
#*******************************************************
#
proc tku_popup_log_open {NAME {ATTRIBUTES ""}} {
   global TKU_G
   set WF .popup_log
   append WF _ $TKU_G(WCOUNTER)
   incr TKU_G(WCOUNTER)

   set WFT $WF.text
   set WFB $WF.button
      
   toplevel $WF
   wm title $WF "$NAME"
   frame $WFT -bg [c_lblue]
            
   text $WFT.text -yscrollcommand "$WFT.scroll set" -setgrid 1 \
        -font [cbr14] -bg [c_yellow] 
   
   set CONF_CMD [list $WFT.text configure]
   foreach WORD $ATTRIBUTES {
      lappend CONF_CMD $WORD
   }
   eval $CONF_CMD
   
   scrollbar $WFT.scroll -command "$WFT.text yview" -bg [c_yellow]
   pack $WFT.scroll -side right -fill y 
   pack $WFT.text -expand 1 -fill both

   $WFT.text tag configure error  -foreground [c_red]
   $WFT.text tag configure normal -foreground [c_black]
   $WFT.text tag configure title \
      -foreground [c_black] -underline 1 -font [cbr18]
   $WFT.text tag configure underline \
      -foreground [c_black] -underline 1 -font [cbr14]
      
   frame $WFB -bg [c_dblue]
   button $WFB.b1 -text "Close" -font [tbr14] -bg [c_aqua] \
          -highlightbackground [c_yellow] \
          -command [list destroy $WF]
   button $WFB.b2 -text "Copy" -font [tbr14] -bg [c_aqua] \
          -highlightbackground [c_yellow] \
          -command [list tku.popup_log_copy $WF $WFT.text]
   pack $WFB.b1 $WFB.b2 -side left -expand 1
   pack $WFT -fill both  -expand 1  
   pack $WFB -side bottom -fill both -expand 1
      
   return "$WF $WFT.text" 
}

proc tku_popup_log {W  TAG MESSAGE} {
   $W insert end $MESSAGE $TAG
}

proc tku_popup_log_show {W} {
   $W.text.text configure -state disabled
   wm transient $W .
   tku_center_on_window . $W
   wm deiconify $W 
}
#.....................................................................
proc tku.popup_log_copy { WF WFT } {
   global TKU_G
   
   if { [string length $TKU_G(COPY_DIR)] == 0 } {
     set TKU_G(COPY_DIR) [tclu_get_tmp_dir]
   }
  tku_cursor_watch       
  tku_restore_tk_getOpenFile 
  set DEST_PATH \
     [tk_getSaveFile -initialdir  $TKU_G(COPY_DIR)]
  tku_override_tk_getOpenFile 
  tku_cursor_normal      
  if { [string length $DEST_PATH] > 0 } { 
       set TKU_G(COPY_DIR) [file dirname $DEST_PATH]
       if [ file exists $DEST_PATH ] {
	  if [ tclu_is_path_dir $DEST_PATH ] {
	     tku_popup_message error ok \
	       "The destination must be a file and\
	      \nnot a directory."
	      return 
	  }
       }
       set MESSAGE "Copy window data to $DEST_PATH ? "
       set CHOICE [tku_popup_message question okcancel $MESSAGE]
       if { $CHOICE != "ok" } {
          return
       }
       if {[catch {open $DEST_PATH w} OFP]} {
         tku_popup_message error ok $OFP
         return 1
       }
       foreach { KEY VALUE INDEX } [ $WFT dump -text 1.0 end ] {
         puts -nonewline $OFP $VALUE
       }
       close $OFP
  }
}

#*******************************************************
#
# Name :	tku_is_root
#
# Description :	
#   Shows a popup alerting that the user is not a superuser (root)
#
# Input	:	MODE - error - if not root - cannot continue
#                      warning - if not root - give a choice
# Output :	None 
# Return Value:	1 if user is root
#               0 otherwise
#*******************************************************
#
proc tku_is_root { MODE } {
   global TKU_G
   
   set IS_ROOT [tclu_is_root]
   
   if { $TKU_G(OS) == "nt" } {
     if { ! $IS_ROOT} {
	set CHOICE [tku_popup_message warning yesno \
                "Administrator privileges are required for \
	       \nperforming this operation.\
	       \nDo you wish to continue ?"]
	if { $CHOICE == "yes" } {
	       tclu_set_root 1
	       return 1
	} else {
	       return 0
	}       
     } else {
       return 1 
     }
   } else {
     if { ! $IS_ROOT} {
       switch -- $MODE {
	 "error" {	 
           tku_popup_message error ok \
             "You do not have superuser privileges\
            \nwhich are required for this operation.\
            \nPlease contact your administrator."
           return 0
	 }
	 "warning" {
	    set CHOICE [tku_popup_message warning yesno \
                "You do not have superuser privileges\
               \nwhich are required for this operation.\
	       \nYou may encounter errors.\
	       \nDo you wish to continue ?"]
	    if { $CHOICE == "yes" } {
	       return 1
	    } else {
	       return 0
	    }
	 }
         default { bgerror "Switch error tku.tcl (2)" }
      }
     } else {
       return 1 
     }
   }
}

#*******************************************************
#
# Name :	tku_prompt_size
#
# Description :	
#   Shows a popup alerting about the size required for the 
#   coming installation
#
# Input	:	SIZE - The required size
# Output :	None 
# Return Value:	1 if user approved
#               0 otherwise
#*******************************************************
#
proc tku_prompt_size { SIZE } {
   if { $SIZE == 0 } {
     return 1
   }
   set APP_SIZE [ expr $SIZE / 1024 ] 
   set CHOICE [tku_popup_message question yesno \
                "The installation you are about to perform\
               \nrequires approximately $APP_SIZE Kbytes.\
	       \nDo you wish to continue ?"]
   if { $CHOICE == "yes" } {
     return 1
   } else {
     return 0
   }
}

################ POPUP MESSAGE #########################
#*******************************************************
#
# Name :	tku_popup_message
#
# Description :	
#   Shows a dialog popup with a message and action buttons.
#   IS better than the Tk default one since it changes colors 
#   depending on type and also does not clip the message 
#   arbitrarily.
#
# Input	:	ICON - error, warning question info
#               TYPE - ok, okcancel retrycancel yesno yesnocancel
#               MESSAGE - Message to display
#               FILE_PATH - A disk file to be displayed in scrolled text
#		ENTRY_VAR_P - A variable to be supplied by the user
# Output :	None 
# Return Value:	None
#
#*******************************************************
#
proc tku_popup_message {ICON TYPE MESSAGE {FILE_PATH ""} {ENTRY_VAR_P ""} } {
   global TKU_G
   
   upvar $ENTRY_VAR_P ENTRY_VAR
   set WF .popup_message
   append WF _ $TKU_G(WCOUNTER)
   incr TKU_G(WCOUNTER)
   set COLORS(error)    [list [c_black] [c_red]]
   set COLORS(warning)  [list [c_black] [c_eggyellow]]
   set COLORS(question) [list [c_black] [c_dblue]]
   set COLORS(info)     [list [c_black] [c_aqua]]
   
   set UPPER 1
   switch -- $TYPE {
      "abortretryignore" {
          set BUTTONS [list abort retry ignore]
      }
      "ok" {
          set BUTTONS [list ok]
      }
      "okcancel" {
          set BUTTONS [list ok cancel]
      }
      "retrycancel" {
          set BUTTONS [list retry cancel]
      }
      "yesno" {
          set BUTTONS [list yes no]
      }
      "yesnocancel" {
          set BUTTONS [list yes no cancel]
      }
      "yesnoshowmore" {
          set BUTTONS [list yes no "show more"]
      }
      default {
	  set BUTTONS $TYPE
	  set UPPER 0
      }
   }
   
   set FG [lindex $COLORS($ICON) 0] 
   set BG [lindex $COLORS($ICON) 1] 
   set WFL $WF.label
   set WFB $WF.button_frame
      
   toplevel $WF -bg $BG
   wm title $WF "Popup Message"
   label $WFL -fg $FG -bg $BG -font [cbr14] -text "$MESSAGE" \
              -relief groove -bd 5 -padx 30 -pady 30
   
   set WFF ""
   if { $FILE_PATH != "" } {
      set WFF $WF.file
      frame $WFF -bg $BG
      text $WFF.text -yscrollcommand "$WFF.scroll set" -setgrid 1 \
        -font [cbr14] -bg [c_yellow] -height 15   
      scrollbar $WFF.scroll -command "$WFF.text yview" -bg [c_yellow]
      pack $WFF.scroll -side right -fill y 
      pack $WFF.text -expand 1 -fill both
      if { ! [ catch {open $FILE_PATH} FP ] } {
	 set FILE_DATA [read $FP]
	 close $FP
         $WFF.text insert end $FILE_DATA    
      }
   }
   set WFE ""
   if { $ENTRY_VAR_P != "" } {
      set WFE $WF.entry
      set TKU_G(ENTRY_VAR) $ENTRY_VAR 
      entry $WFE -bg [c_yellow] -font [cbr14] \
           -textvariable TKU_G(ENTRY_VAR) -width 30
   }
   
   frame $WFB -bg $BG
   set I 0 
   foreach BUTTON $BUTTONS {
      set WB $WFB.button$I
      set STRING $BUTTON
      if { $UPPER } {
	 set STRING [string toupper $BUTTON]
      }
      button $WB -text $STRING -bg [c_aqua] -font [tbr18] \
             -highlightbackground $BG \
             -command [list tku.popup_message_cb $WF $BUTTON]
      pack $WB -side left -expand 1
      incr I
   }
   pack $WFL -fill x -padx 10 -pady 10
   if { $WFF != "" } {
      pack $WFF -fill x -padx 10 -pady 10
   }
   if { $WFE != "" } {
      pack $WFE -fill x -padx 10 -pady 10
   }
   pack $WFB -fill x -padx 10 -pady 10
   set TKU_G(MESSAGE_RES) ""
   wm transient $WF .
   tku_center_on_window . $WF 
   wm deiconify $WF
   focus $WF
   set TKU_G(GRAB_CURRENT) [grab current $WF]
   catch {grab $WF}
   tkwait window $WF
   if { [string length $TKU_G(MESSAGE_RES)] == 0 } {
      bgerror "Illegal popup destruction"
   }
   if { $WFE != "" } {
      set ENTRY_VAR $TKU_G(ENTRY_VAR) 
   }
   
   return $TKU_G(MESSAGE_RES)
}

proc tku.popup_message_cb {WF BUTTON} {
   global TKU_G
   
   set TKU_G(MESSAGE_RES) $BUTTON
   destroy $WF
   update
   grab release $WF
   if { [string length $TKU_G(GRAB_CURRENT)] != 0 } {
      grab set $TKU_G(GRAB_CURRENT)
   }
}


################ SCROLLED LISTS #########################
proc tku_scrolled_listbox { W ARGS_LIST ARGS_SB } {
   frame $W
   listbox $W.list \
     -xscrollcommand [list tku.scroll_set $W.xscroll \
        [list grid $W.xscroll -row 1 -column 0  -sticky we]] \
     -yscrollcommand [list tku.scroll_set $W.yscroll \
        [list grid $W.yscroll -row 0 -column 1 -sticky ns]]
   eval {$W.list configure} $ARGS_LIST
   
   scrollbar $W.xscroll -orient horizontal \
      -command [list $W.list xview]
   eval {$W.xscroll configure} $ARGS_SB
   
   scrollbar $W.yscroll  -orient vertical \
      -command [list $W.list yview]
   eval {$W.yscroll configure} $ARGS_SB
   
   grid $W.list $W.yscroll -sticky news
   grid $W.xscroll -sticky news
   grid rowconfigure $W 0 -weight 1
   grid columnconfigure $W 0 -weight 1
   return  $W.list 
}

proc tku.scroll_set { SCROLLBAR GEOCMD OFFSET SIZE } {
   if { $OFFSET != 0.0 || $SIZE != 1.0 } {
     eval $GEOCMD
     $SCROLLBAR set $OFFSET $SIZE
   } else {
     set MANAGER [lindex $GEOCMD 0]
     $MANAGER forget $SCROLLBAR 
   }
}
################  CURSORS #############################
proc tku_cursor_watch {} {
   . configure -cursor watch
   update idletasks
}

proc tku_cursor_normal {} {
   . configure -cursor left_ptr
   update idletasks
}
################  GEOMETRY #############################
#*******************************************************
#
# Name :	tku_center
#
# Description :	
#   Gets the center of a window and format it into a geometry string
#
# Input	:	toplevel window
#               XOFFSET - value to subtract from xcenter
#               YOFFSET - value to subtract from ycenter
# Output :	None
# Return Value:	+x+y
#
#*******************************************************
#
proc tku_center {W {XOFFSET 0} {YOFFSET 0}} {
  set GEOM [wm geometry $W]
  scan $GEOM "%dx%d+%d+%d" W H X Y
  set CX [expr $X+$W/2-$XOFFSET] 
  set CY [expr $Y+$H/2-$YOFFSET]
  return "+$CX+$CY"
}

#*******************************************************
#
# Name :	tku_center_on_window
#
# Description :	
#   Position window W2 exactly at the center of window W1
#
# Input	:	toplevel window
#               XOFFSET - value to subtract from xcenter
#               YOFFSET - value to subtract from ycenter
# Output :	None
# Return Value:	+x+y
#
#*******************************************************
#
proc tku_center_on_window {W1 W2} {
  set GEOM [wm geometry $W1]
  scan $GEOM "%dx%d+%d+%d" W H X Y
  set CX [expr $X+$W/2-200] 
  set CY [expr $Y+$H/2-200]
  set CENTER "+$CX+$CY"
  wm geometry $W2 $CENTER
  tkwait visibility $W2
  
  set WIDTH  [winfo width $W2]
  set HEIGHT [winfo height $W2]
  
  set XOFFSET [expr $WIDTH/2 -$W/2]
  set YOFFSET [expr $HEIGHT/2-$H/2]
  
  if { $XOFFSET > $X } {
     set XOFFSET $X
  }
  if { $YOFFSET > $Y } {
     set YOFFSET $Y
  }
  set CX [expr $X-$XOFFSET] 
  set CY [expr $Y-$YOFFSET]
  set CENTER "+$CX+$CY"
  wm geometry $W2 $CENTER
  wm deiconify $W2
  return $CENTER
}

################  COLOR #############################
#*******************************************************
#
# Name :	tku_color
#
# Description :	
#   Converts rrggbb to Tk color scheme
#
# Input	:	rrggbb
# Output :	None
# Return Value:	Tk Color
#
#*******************************************************
#
proc tku_color {RGB} {
  set R [expr $RGB/10000]
  set G [expr ($RGB - $R*10000)/100]
  set B [expr $RGB % 100]

  set R [expr $R*256/99]
  set R [ expr $R < 255 ? $R : 255 ]
  set G [expr $G*256/99]
  set G [ expr $G < 255 ? $G : 255 ]
  set B [expr $B*256/99]
  set B [ expr $B < 255 ? $B : 255 ]

  set COLOR [format "#%02x%02x%02x" $R $G $B]
  return $COLOR
}

proc c_white     {} {tku_color 999999}
proc c_black     {} {tku_color 000000}
proc c_gray      {} {tku_color 707070}
proc c_lblue     {} {tku_color 709999}
proc c_dblue     {} {tku_color 608585}
proc c_aqua      {} {tku_color 328068}
proc c_yellow    {} {tku_color 999980}
proc c_red       {} {tku_color 992020}
proc c_eggyellow {} {tku_color 987730}
proc c_mviolet   {} {tku_color 706080}

##############  FONT ##############################
#*******************************************************
#
# Name :	tku_font
#
# Description :	
#   Converts the font in xyznn to Tk 8.0 standard font
#
# Input	:	FN - xyznn
#               UNDERLINE - u for underlining
# Output :	None
# Return Value:	Tk standard font {type size {styles}}
#
#*******************************************************
#

proc tku_font {FN {UNDERLINE n}} {
  tk scaling 1
  set TYPE   [string index $FN 0]
  set BOLD   [string index $FN 1]
  set ITALIC [string index $FN 2]
  set SIZE   [string range $FN 3 4]

  set FTYPE   courier
  set FBOLD   bold
  set FITALIC regular
  set FUNDERLINE ""
  switch -- $TYPE {
    t { set FTYPE times }
    h { set FTYPE helvetica }
    c { set FTYPE courier }
  }
  switch -- $BOLD {
    b { set FBOLD bold }
    m { set FBOLD normal }
  }
  switch -- $ITALIC {
    r { set FITALIC roman }
    i { set FITALIC italic }
  }
  set STYLE [list $FBOLD $FITALIC]
  if { $UNDERLINE == "u" } {
     lappend STYLE underline
  }

  set FONT [list $FTYPE $SIZE $STYLE]
  return $FONT
}

proc tbr14 {} {return [tku_font tbr14]}
proc tbr18 {} {return [tku_font tbr18]}
proc tbr24 {} {return [tku_font tbr24]}

proc tbi14 {} {return [tku_font tbi14]}
proc tbi18 {} {return [tku_font tbi18]}
proc tbi24 {} {return [tku_font tbi24]}

proc tmr14 {} {return [tku_font tmr14]}
proc tmr18 {} {return [tku_font tmr18]}
proc tmr24 {} {return [tku_font tmr24]}

proc tmi14 {} {return [tku_font tmi14]}
proc tmi18 {} {return [tku_font tmi18]}
proc tmi24 {} {return [tku_font tmi24]}

proc cbr14 {} {return [tku_font cbr14]}
proc cbr18 {} {return [tku_font cbr18]}
proc cbr24 {} {return [tku_font cbr24]}

proc cbi14 {} {return [tku_font cbi14]}
proc cbi18 {} {return [tku_font cbi18]}
proc cbi24 {} {return [tku_font cbi24]}

proc cmr14 {} {return [tku_font cmr14]}
proc cmr18 {} {return [tku_font cmr18]}
proc cmr24 {} {return [tku_font cmr24]}

proc cmi14 {} {return [tku_font cmi14]}
proc cmi18 {} {return [tku_font cmi18]}
proc cmi24 {} {return [tku_font cmi24]}

proc hbr14 {} {return [tku_font hbr14]}
proc hbr18 {} {return [tku_font hbr18]}
proc hbr24 {} {return [tku_font hbr24]}

proc hbi14 {} {return [tku_font hbi14]}
proc hbi18 {} {return [tku_font hbi18]}
proc hbi24 {} {return [tku_font hbi24]}

proc hmr14 {} {return [tku_font hmr14]}
proc hmr18 {} {return [tku_font hmr18]}
proc hmr24 {} {return [tku_font hmr24]}

proc hmi14 {} {return [tku_font hmi14]}
proc hmi18 {} {return [tku_font hmi18]}
proc hmi24 {} {return [tku_font hmi24]}

