#!/bin/csh
# File Name:    script_start.csh
# File Purpose: This file is a c-shell script which is sourced by the system
#               everytime any c-shell script is run. The function of this
#               hook is to hold all the standard script variables and aliases
#               that may be called / used by c-shell scripts.
# Written by:   Ian Ticehurst (Autom8 Ltd.)
# Version 1A.:  Date: 14.08.00 - Initial design and development.
# Version 1B.:  Date: 04.09.00 - Addition of awk, sort and find aliases for NT (ref. CB-Orbotech).
# Version 1C.:  Date: 26.10.00 - Addition of $TMP defintion and useage.
# Version 1D.:  Date: 05.12.00 - Removal of "gui" alias and "gui" data and response settings (set locally).
# Version 1E.:  Date: 06.12.00 - Added TCSHSUBSTHB definition to allow calling tcl scripts on NT from tcsh 5.44
############################# Start of notes ###############################
# This file is called automatically everytime a c-shell script is run. Thus,
# by adding aliases, path definitions, and global variable statements etc.
# every c-shell script that is called within the Genesis environment can use
# these values. So, instead of adding these definitions to each individual
# script, the definitions appear only in one place.
############################# End of notes ###############################

# Define correct temp. dir. depending on env. vars.
if ($?GENESIS_TMP) then
	set TMP = $GENESIS_TMP
else if ($?GENESIS_DIR) then
	set TMP = $GENESIS_DIR/tmp
else
	set TMP = /genesis/tmp
endif

# Detect current operating system.
set CURR_OS = `uname -s`

# Set the INFO alias and the tmp INFO file.
set IFILE = $TMP/do_info.$$
alias DO_INFO 'COM info,out_file=$IFILE,write_mode=replace,args= \!:* ; source $IFILE; rm $IFILE'

# If working on NT machine, set awk, sort and find aliases.
if ($CURR_OS == "WINDOWS_NT") then
	alias awk 'c:/bin/awk95'
	alias sort 'c:/usr/local/bin/sort'
	alias find 'c:/usr/local/bin/find'
	# Modify if tclsh80.exe is found elsewhere or if wish_start.tcl has been modified
	setenv TCSHSUBSTHB "/usr/local/bin/wish c:/progra~1/tcl/bin/tclsh80.exe;"
endif

# Set up alias for using bc in numeric calculations (this MATH alias can be used in the same way as "set" and "@" are used).
# (for more information, check the man pages for "bc").
alias MATH 'set \!:1 = `echo "\!:3-$" | bc -l`'

exit
